#ifndef _DATEXIT_H
#define _DATEXIT_H

// This file was automatically generated by objch.

//==============================================================================
//
//           DAtExit - the "at-exit" class in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-24 05:37:06 $ $Revision: 1.4 $
//
//==============================================================================

#include "ofc/config.h"

//
// The AtExit implements a number of methods for insuring the freeing of 
// objects during the exit of the program. This class has only class methods
// (singleton).
//
// @example
// #include <stdio.h>
// #include "ofc/DAtExit.h"
// #include "ofc/DFile.h"
// 
// int main(int argc, char *argv[])
// {
//   DFile *file = [DFile new];
//   
//   if ([file open :"example.ini" :"r"])
//   {
//     [DAtExit add :file];       // If program exit, then automatically call [file free]
// 
//     // Read the file ..
// 
//     [file close];
// 
//     [DAtExit remove :file];    // If program exit, [file free] is not called anymore
//   }
// 
//   [file free];
// 
//   return 0;
// }
// 

@interface DAtExit : Object
{
}

//
// Add an object that must be freed during program exit
//
// @param obj      the object
// 
// @return none
//
+ (void) add :(id) obj;

//
// Remove an object from freeing during program exit
//
// @param obj      the object
//
// @return none
//
+ (void) remove :(id) obj;

@end
#endif
