#ifndef _DCRC32_H
#define _DCRC32_H

// This file was automatically generated by objch.

//==============================================================================
//
//    DCRC32 - the cyclic redundancy check class in the ofc-library
//
//               Copyright (C) 2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-26 05:50:30 $ $Revision: 1.6 $
//==============================================================================

#include <stdint.h>

#include "ofc/config.h"
//
// The CRC32 class implements a number of methods for calculating the CRC-32
// checksum.
//
// @example
//
// #include <stdio.h>
// #include "ofc/DCRC32.h"
// 
// int main(int argc, char *argv[])
// {
//   DCRC32 *crc = [DCRC32 new];
// 
//   [crc update :"This is an example for the CRC32 class"];  // Feed the CRC32 with a c-string
// 
//   [crc update :"This is data\0With string terminator" :35]; // Feed the CRC32 with data
// 
//   printf("CRC32 value: %lx.\n", [crc crc32]);              // Get the CRC32 value
// 
//   [crc free];                                              // Cleanup
// 
//   return 0;
// }
// 

@interface DCRC32 : Object
{
@private
  uint32_t _crc32;    // the crc32 value
}


//// Constructors

//
// Initialise to an empty object
//
// @return the instance
//
- (DCRC32 *) init;

//
// Initialise the object with a c-string
//
// @param cstring  the string
//
// @return the object
//
- (DCRC32 *) init :(const char *) cstring;

//
// Initialise the object with data
//
// @param data     the data ('\0' allowed)
// @param length   the length of the data
//
// @return the object
//
- (DCRC32 *) init :(const unsigned char *) data :(unsigned long) length;


//// Update methods

//
// Update the object with a c-string. Repeated calls can be compared
// to a single call in which the strings are concatted
//
// @param cstring  the c-string to be feeded to crc32
//
// @return the object
//
- (DCRC32 *) update :(const char *) cstring;

//
// Update the object with data. Repeated calls can be compared
// to a single call in which the strings are concatted
//
// @param data     the data to be feeded to md5
// @param length   the length of the data
//
// @return the object
//
- (DCRC32 *) update :(const unsigned char *) data :(unsigned long) length;


//// Result methods

//
// Return the (current) result of all data passed to the object. 
//
// @return the crc32 value
//
- (unsigned long) crc32;

@end
#endif
