#ifndef _DDATETIME_H
#define _DDATETIME_H

// This file was automatically generated by objch.

//==============================================================================
//
//          DDateTime - the date & time datatype in the ofc-library
//
//               Copyright (C) 2004  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-07-20 14:03:51 $ $Revision: 1.9 $
//
//==============================================================================
#ifdef WIN32
#endif

#include "ofc/config.h"

#include "ofc/DText.h"
#include "ofc/DTextable.h"
#include "ofc/DParsable.h"
#include "ofc/DComparable.h"

#define DDT_SUNDAY      (0)
#define DDT_MONDAY      (1)
#define DDT_TUESDAY     (2) 
#define DDT_WEDNESDAY   (3)
#define DDT_THURSDAY    (4)
#define DDT_FRIDAY      (5)
#define DDT_SATURDAY    (6)

#define DDT_MIN_YEAR    (1970)

//
// The DDateTime class implements a date & time datatype.
//
// @example
// #include <stdio.h>
// #include "ofc/DDateTime.h"
// 
// int main(int argc, char *argv[])
// {
//   DDateTime *dtm1 = [DDateTime alloc];
//   DDateTime *dtm2 = [DDateTime new  ];
//   DText     *str;
// 
//   [dtm1 init :2008 :7 :20 :15 :3 :45]; // Init with a date and time
// 
//   [dtm2 time :16 :50 :8];              // Set the time
//   [dtm2 date :2007 :12 :31];           // Set the date
// 
//   str = [dtm1 toISO8601];
//   printf("Date1 in ISO8601 format:%s.\n", [str cstring]);
//   [str free];
// 
//   [dtm2 localTime];                    // Set the date&time with localtime
// 
//   str = [dtm2 toASC];
//   printf("LocalTime in asctime format:%s.\n", [str cstring]);
//   [str free];
// 
//   [dtm2 UTCTime];                      // Set the date&time with UTC time
//   str = [dtm2 format :"%a, %d-%b-%Y %H:%M:%S %z"];
//   printf("UTCTime with format:%s.\n", [str cstring]);
//   [str free];
// 
//   if ([dtm1 compare :dtm2] == 0)       // Compare two dates
//     printf("Date1 is equal to date2.\n");
//   else if ([dtm1 compare :dtm2] < 0)
//     printf("Date1 is smaller than date2.\n");
//   else
//     printf("Date1 is greater than date2.\n");
// 
//   [dtm1 free];                         // Cleanup
//   [dtm2 free];
// 
//   return 0;
// }
// 

@interface DDateTime : Object <DTextable,DParsable,DComparable>
{
@private
  int                 _millis;    // the milliseconds (0..999)
  int                 _seconds;   // the seconds (0..59)
  int                 _minutes;   // the minutes (0..59)
  int                 _hours;     // the hours (0..23)
  int                 _day;       // the day (1..31)
  int                 _month;     // the month (1..12)
  int                 _year;      // the year (1970..)
  int                 _weekday;   // the week day 0..6
}


//// Constructors

//
// Initialise the object with a default date
//
// @return the object
//
- (DDateTime *) init;

//
// Initialise with a date/time
//
// @param year     the year (1970..)
// @param month    the month (1..12)
// @param day      the day (1..31)
// @param hours    the hours (0..23)
// @param minutes  the minutes (0..59)
// @param seconds  the seconds (0..59)
// 
// @return the object
//
- (DDateTime *) init :(int) year :(int) month :(int) day :(int) hours :(int) minutes :(int) seconds;


//// Member methods

//
// Return the year
// 
// @return the year (1970..)
//
- (int) year;

//
// Return the month
// 
// @return the month (1..12)
// 
- (int) month;

//
// Return the day
// 
// @return the day (1..31)
// 
- (int) day;

//
// Return the hours
// 
// @return the hours (0..23)
// 
- (int) hours;

//
// Return the minutes
// 
// @return the minutes (0..59)
// 
- (int) minutes;

//
// Return the seconds
// 
// @return the seconds (0..59)
// 
- (int) seconds;

//
// Return the milliseconds
// 
// @return the milliseconds (0..999)
// 
- (int) millis;

//
// Return the day of the week 
// 
// @return the day of the week (0..6)
// 
- (int) weekday;


//// Class methods

//
// Check if a year is a leap year
//
// @param year     the year to be checked
//
// @return is it ?
//
+ (BOOL) isLeapYear :(int) year;

//
// Return the number of days in a month
//
// @param year     the year of the month
// @param month    the month (1..12)
//
// @return the number of days in the month (1..31) or (-1)
//
+ (int) daysInMonth :(int) year :(int) month;

//
// Check if the date is valid
// 
// @param year     the year (1970..)
// @param month    the month (1..12)
// @param day      the day (1..31)
// 
// @return is it ?
// 
+ (BOOL) isDateValid :(int) year :(int) month :(int) day;

//
// Check if the time is valid
// 
// @param hours    the hours (0..23)
// @param minutes  the minutes (0..59)
// @param seconds  the seconds (0..59)
// @param millis   the milliseconds (0..999)
// 
// @return is it ?
// 
+ (BOOL) isTimeValid :(int) hours :(int) minutes :(int) seconds :(int) millis;

//
// Check if the date and time are valid
// 
// @param year     the year (1970..)
// @param month    the month (1..12)
// @param day      the day (1..31)
// @param hours    the hours (0..23)
// @param minutes  the minutes (0..59)
// @param seconds  the seconds (0..59)
// @param millis   the milliseconds (0..999)
// 
// @return is it ?
// 
+ (BOOL) isValid :(int) year :(int) month :(int) day :(int) hours :(int) minutes :(int) seconds :(int) millis;


//// Set methods

// 
// Set the date/time
//
// @param year     the year (1970..)
// @param month    the month (1..12)
// @param day      the day (1..31)
// @param hours    the hours (0..23)
// @param minutes  the minutes (0..59)
// @param seconds  the seconds (0..59)
// 
// @return success
// 
- (BOOL) set :(int) year :(int) month :(int) day :(int) hours :(int) minutes :(int) seconds;

// 
// Set the date/time
//
// @param year     the year (1970..)
// @param month    the month (1..12)
// @param day      the day (1..31)
// @param hours    the hours (0..23)
// @param minutes  the minutes (0..59)
// @param seconds  the seconds (0..59)
// @param millis   the milliseconds (0..999)
// 
// @return success
// 
- (BOOL) set :(int) year :(int) month :(int) day :(int) hours :(int) minutes :(int) seconds :(int) millis;

// 
// Set the time
//
// @param hours    the hours (0..23)
// @param minutes  the minutes (0..59)
// @param seconds  the seconds (0..59)
// 
// @return success
// 
- (BOOL) time :(int) hours :(int) minutes :(int) seconds;

// 
// Set the time (incl. milliseconds)
//
// @param hours    the hours (0..23)
// @param minutes  the minutes (0..59)
// @param seconds  the seconds (0..59)
// @param millis   the milliseconds (0..999)
// 
// @return success
// 
- (BOOL) time :(int) hours :(int) minutes :(int) seconds :(int) millis;

// 
// Set the date
//
// @param year     the year (1970..)
// @param month    the month (1..12)
// @param day      the day (1..31)
// 
// @return success
// 
- (BOOL) date :(int) year :(int) month :(int) day;

// 
// Set the object to the current local date/time
//
// @return success
// 
- (BOOL) localTime;

#ifdef WIN32
#endif
// 
// Set the object to the current UTC date/time
//
// @return success
// 
- (BOOL) UTCTime;

#ifdef WIN32
#endif
// 
// Normalize the object
// 
// @return success
// 
- (BOOL) norm;


//// String output methods

//
// Format the date/time as specified in ISO 8601 (without the milliseconds and 'T' as separator)
// 
// @return a (new) text string object
// 
- (DText *) toISO8601;

//
// Format the date/time as specified in RFC1123
// 
// @return a (new) text string object (or nil)
//
- (DText *) toRFC1123;

//
// Format the date/time as specified in RFC850
// 
// @return a (new) text string object (or nil)
// 
- (DText *) toRFC850;

//
// Format the date/time as specified in RFC822
// 
// @return a (new) text string object (or nil)
// 
- (DText *) toRFC822;

//
// Format the date/time as  returned by asctime
// 
// @return a (new) text string object (or nil)
// 
- (DText *) toASC;

//
// Format the date/time accordingly a format string (see strftime)
// 
// @param format    the format c-string
// 
// @return a (new) text string object (or nil)
// 
- (DText *) format :(const char *) format;


//// DTextable protocol implementation

//
// Format the date/time object as text (ISO 8601 format with milliseconds and ' ' as separator)
// 
// @return a (new) text string object
//
- (DText *) toText;


//// DParsable protocol implementation

//
// Parse a string to set a date time object.
// This method supports the following format: <br>
//  yyyy-mm-dd hh:mm:ss.mmm -- date/time accordingly ISO 8061 <br>
//  milliseconds are optional, also the separators
//
// @param cstr     the string to be parsed (moved to first non-parsable char)
// 
// @return the result (0, ERANGE, ENODATA)
//
- (int) fromString :(char **) cstr;

//
// Parse a string to set the object with a date.
// This method supports the following format: (ISO 8061)<br>
//  yyyy-mm-dd yy-mm-dd yyyymmdd yymmdd
//
// @param cstr     the string to be parsed (moved to first non-parsable char)
// 
// @return the result (0, ERANGE, ENODATA)
//
- (int) date :(char **) cstr;

//
// Parse a string to set a date time object with a time.
// This method supports the following format: (ISO 8061)<br>
//  hh:mm:ss.mmm hh:mm:ss hhmmss.mmm hhmmss hh:mm hhmm
//
// @param cstr     the string to be parsed (moved to first non-parsable char)
// 
// @return the result (0, ERANGE, ENODATA)
//
- (int) time :(char **) cstr;


//// String parse methods (requires strptime)

#ifdef HAVE_STRPTIME
#endif
//
// Parse a string with a format string to set the date/time object (requires strptime)
// 
// @param cstr     the string to be parsed (moved to the first non-parsable char)
// @param format   the format for the string (see strptime)
// 
// @return is the string parsed ?
// 
- (BOOL) parse :(char **) cstr :(const char *) format;

#ifdef HAVE_STRPTIME
#endif
//
// Parse the string as specified in RFC1123 to set the date/time object (requires strptime)
//
// @param cstr     the string to be parsed (moved to first non-parsable char)
// 
// @return is the string parsed ?
//
- (BOOL) fromRFC1123 :(char **) cstr;

//
// Parse the string as specified in RFC850 to set the date/time object (required strptime)
// 
// @param cstr     the string to be parsed (moved to first non-parsable char)
// 
// @return is the string parsed ?
// 
- (BOOL) fromRFC850 :(char **) cstr;

// Parse the string as specified in RFC822 to set the date/time object (requires strptime)
//
// @param cstr     the string to be parsed (moved to first non-parsable char)
// 
// @return is the string parsed ?
//
- (BOOL) fromRFC822 :(char **) cstr;

//
// Parse the string with the format of asctime to set the date/time object (requires strptime)
// 
// @param cstr     the string to be parsed (moved to first non-parsable char)
// 
// @return is the string parsed ?
// 
- (BOOL) fromASC :(char **) cstr;


//// Comparable protocol implementation

//
// Compare the DateTime object with another DateTime object
//
// @param other    the object that should be compared
//
// @return the compare result (-1, 0, 1)
//
- (int) compare :(DDateTime *) other;

@end
#endif
