#ifndef _DDOUBLE_H
#define _DDOUBLE_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DDouble - the double number class in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-07-24 18:02:46 $ $Revision: 1.8 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DComparable.h"
#include "ofc/DTextable.h"
#include "ofc/DDatable.h"
#include "ofc/DParsable.h"

#include "ofc/DText.h"
#include "ofc/DData.h"

//
// The Double class implements a number of methods for storing and manipulating double numbers.
//
// @example
// #include <stdio.h>
// #include "ofc/DDouble.h"
// 
// int main(int argc, char *argv[])
// {
//   DDouble *db1  = [DDouble alloc];
//   DDouble *db2  = [DDouble new  ];
//   DText   *str;
// 
//   [db1 init :7.0];                     // Init with a value
//   
//   [db2 set :14.0];                     // Set with a value
// 
//   str = [db1 toText];
//   printf("Double1 is:%.1f and \"%s\".\n", [db1 get], [str cstring]);
//   [str free];
// 
//   if ([db1 compare :db2] == 0)
//     printf("Double1 is equal to double2.\n");
//   else
//     if ([db1 compare :db2] < 0)
//       printf("Double1 is smaller than double2.\n");
//     else
//       printf("Double1 is greater than double2.\n");
// 
//   [db1 free];                          // Cleanup
//   [db2 free];
// 
//   return 0;
// } 
//   

@interface DDouble : Object <DComparable,DTextable,DDatable,DParsable>
{
@private
  double       _double;
}


//// Constructors

//
// Initialise to number zero
//
// @return the object
//
- (DDouble *) init;

//
// Initialise to a number
//
// @param number   the number
//
// @return the object
//
- (DDouble *) init :(double) number;


//// Member methods

//
// Get the double
//
// @return the double number
//
- (double) get;

//
// Set the double number
//
// @param number  the double number
//
// @return the object
//
- (DDouble *) set :(double) number;


//// Comparable protocol

//
// Compare two double objects
//
// @param other    the object to compare with
//
// @return the result of the comparison (-1,0,1)
//
- (int) compare :(DDouble *) other;


//// Textable protocol

//
// Return a text string with the double
//
// @return a (new) text string with the number
//
- (DText *) toText;


//// Datable protocol

//
// Return a data string with the double
//
// @return a (new) data string
//
- (DData *) toData;


//// Parsable protocol

//
// Parse a string to set the double
//
// @param cstr     the string to be parsed (moved to first non-parsable char)
// 
// @return the result (0, ERANGE, ENODATA)
//
- (int) fromString :(char **) cstr;

@end
#endif
