#ifndef _DFILE_H
#define _DFILE_H

// This file was automatically generated by objch.

//==============================================================================
//
//              DFile - the file class in the ofc-library
//
//             Copyright (C) 2003-2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-24 05:37:06 $ $Revision: 1.23 $
//
//==============================================================================
#ifdef WIN32
#endif

#include <stdio.h>

#include "ofc/config.h"

#include "ofc/DText.h"
#include "ofc/DData.h"
#include "ofc/DList.h"
#include "ofc/DTextReadable.h"
#include "ofc/DTextWritable.h"
#include "ofc/DDataReadable.h"
#include "ofc/DDataWritable.h"
#include "ofc/DDateTime.h"

#define DFILE_SEEK_SET  0
#define DFILE_SEEK_CUR  1
#define DFILE_SEEK_END  2

//
// The DFile class implements a number of methods for opening of, writing to, 
// reading from and closing of files.
//
// @example
// #include <stdio.h>
// #include "ofc/DFile.h"
// 
// int main(int argc, char *argv[])
// {
//   DFile *fil1  = [DFile alloc];
//   DFile *fil2  = [DFile new  ];
//   DText *str;
//                                        // Init with a file
//   [fil1 init :"test.txt" :"w"];
// 
//   if ([fil1 isOpen])
//   {
//     printf("File \"test.txt\" is open for writing.\n");
// 
//     if ([fil1 writeLine :"This is a test file for the DFile example."])
//       printf("Text succesfully writen in file.\n");
//     else
//       printf("Text could not be written in file:%d\n", [fil1 error]);
//     
//     [fil1 close];
//   }
//   else
//     printf("File \"test.txt\" could not be opened:%d.\n", [fil1 error]);
// 
//   if ([fil2 open :"test.txt" :"r"])
//   {
//     printf("File \"test.txt\" is open for reading.\n");
// 
//                                        // Read the contents of the text file
//     str = [fil2 readLine];
//     while (str != nil)
//     {
//       printf("Read text:%s\n", [str cstring]);
//       [str free];
//       str = [fil2 readLine];
//     }
// 
//     [fil2 close];                      // Close the file
//   }
//   else
//     printf("File \"test.txt\" could not be opened:%d.\n", [fil2 error]);
// 
//   [fil1 free];                         // Cleanup
//   [fil2 free];
// 
//                                       // Class methods for files
//   if ([DFile move :"test.txt" :"test2.txt"])
//     printf("File \"test.txt\" succesfully renamed to \"test2.txt\".\n");
//   else
//     printf("File \"test.txt\" could not be renamed:%d.\n", [DFile error]);
// 
//   printf("File \"test2.txt\" is %s a directory.\n", ([DFile isDirectory :"test2.txt"] ? "" : "not"));
// 
//   printf("Size of file \"test2.txt\":%lld\n", [DFile size :"test2.txt"]);
// 
//   if ([DFile remove :"test2.txt"])
//     printf("File \"test2.txt\" is succesfully removed.\n");
//   else
//     printf("File \"test2.txt\" could not be removed:%d\n", [DFile error]);
// 
//   return 0;
// }
// 

@interface DFile : Object <DTextReadable,DTextWritable,DDataReadable,DDataWritable>
{
@private
  FILE           *_file;    // the file object
}


//// Constructors

//
// Initialise an empty file object
//
// @return the object
//
- (DFile *) init;

//
// Open a file
//
// @param name     the filename (c-string)
// @param mode     the file mode (c-string) (r,r+,w,w+,a,a+)
//
// @return the object
//
- (DFile *) init :(const char *) name :(const char *) mode;


//// Copy related methods

//
// Do a shallow copy of the object (not implemented)
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the object
//
// @return the file object
//
- free;


//// File info methods

//
// Check if the file is open
//
// @return is the file open ?
//
- (BOOL) isOpen;

//
// Return the last error
//
// @return the last error
//
- (int) error;

//
// Check if the file is an interactive terminal
//
// @return is it ?
//
- (BOOL) isAtty;

//
// Return the file descriptor 
//
// @return the file descriptor (or -1 on error)
//
- (int) fileno;


//// File opening methods

//
// Open a file
//
// @param name     the filename
// @param mode     the file mode (r,r+,w,w+,a,a+)
//
// @return is the file open ?
//
- (BOOL) open :(const char *) name :(const char *) mode;


//// TextReadable protocol implementation

//
// Read a text string (all available text)
//
// @return a (new) text string
//
- (DText *) readText;

//
// Read a text string 
//
// @param length   the length of the text to be read
//
// @return a (new) text string
//
- (DText *) readText :(long) length;

//
// Read a line of text (till Eof or \n)
//
// @return a (new) text string (or nil for no data)
//
- (DText *) readLine;;

//
// Read a character
//
// @return the character read (EOS for EOF)
//
- (char) readChar;;

//
// Tell the current position in the readable
//
// @return the current position (or -1 for error)
//
- (unsigned long) tell;;

//
// Move the current position in the readable
//
// @param offset   the offset from the origin (in bytes)
// @param origin   the origin for the offset (0=begin, 1=current, 2=end)
//
// @return success
//
- (BOOL) seek :(unsigned long) offset :(int) origin;;

//
// Skip a number of positions
//
// @param offset   the number of bytes to skip
//
// @return success
//
- (BOOL) skip :(unsigned long) offset;;

//
// Check if the end of the file is reached
//
// @return is it?
//
- (BOOL) isEof;;


//// Implementation of DTextWritable

//
// Write text string 
//
// @param text     the text to be written
//
// @return success
//
- (BOOL) writeText :(const char *) text;;

//
// Write line (appending a '\n')
//
// @param text     the text to be written
//
// @return success
//
- (BOOL) writeLine :(const char *) text;

//
// Write a character
//
// @param ch       the character to be written
//
// @return success
//
- (BOOL) writeChar :(char) ch;;


//// Methods for DDataReadable protocol

//
// Read a data string 
//
// @param length   the length of the data
//
// @return the object
//
- (DData *) readData :(unsigned long) length;;

//
// Read a data string 
//
// @param dest     the destination buffer
// @param length   the length of the data (and minimum size of dest)
//
// @return the actual number of bytes read
//
- (unsigned long) readData :(unsigned char *) dest :(unsigned long) length;

//
// Read a byte
//
// @return the byte read (0 for error)
//
- (unsigned char) readByte;;

//
// Read a short
//
// @return the short
//
- (short) readShort;;

//
// Read a long
//
// @return the long
//
- (long) readLong;;

//
// Read a double
//
// @return the double
//
- (double) readDouble;;


//// DDataWritable protocol implementation

//
// Write a data string 
//
// @param data     the data to be written
// @param length   the length of the data
//
// @return success
//
- (BOOL) writeData :(const unsigned char *) text :(unsigned long) length;;

//
// Write a byte
//
// @param byte     the byte to be written
//
// @return success
//
- (BOOL) writeByte :(unsigned char) byte;;

//
// Write a short 
//
// @param nr       the short to be written
//
// @return success
//
- (BOOL) writeShort :(short) nr;;

//
// Write a long
//
// @param nr       the long to be written
//
// @return success
//
- (BOOL) writeLong :(long) nr;

//
// Write a double
//
// @param nr       the double to be written
//
// @return success
//
- (BOOL) writeDouble :(double) nr;


//// List related methods

//
// Read all lines from a file into a list of strings
//
// @return a (new) list of (new) DText objects
//
- (DList *) readLines;

//
// Writes all the elements ('lines') of text from a list to file
//
// @param list     the list with objects that responds to textable protocol
//
// @return success
//
- (BOOL) writeLines :(DList *) list;


//// File manipulation

//
// Truncate the file (not implemented for WIN32)
//
// @param size     the location of the truncation (-1 = current file position)
//
// @return success
//
- (BOOL) truncate :(long) size;

#ifndef WIN32
#endif 
//
// Flush the output buffers of the file
//
// @return success
//
- (BOOL) flush;


//// File closing methods

//
// Close the file
//
// @return the file object
//
- close;


//// Class methods

//
// Return the last error for the class methods
// 
// @return the last error
// 
+ (int) error;

//
// Move/Rename a file
// 
// @param path         the current path
// @param newPath      the new path
// 
// @return success
// 
+ (BOOL) move :(const char *) path :(const char *) newPath;

#ifdef WIN32
#endif
// 
// Remove a file
// 
// @param path         the path 
// 
// @return success
// 
+ (BOOL) remove :(const char *) path;

#ifdef WIN32
#endif
//
// Check if the file exists (as file)
// 
// @param path         the path of the file
// 
// @return does it ?
// 
+ (BOOL) isFile :(const char *) path;

#ifdef WIN32
#endif
//
// Check if the file exists as directory
// 
// @param path         the path of the file/directory
// 
// @return does it ?
// 
+ (BOOL) isDirectory :(const char *) path;

#ifdef WIN32
#endif
//
// Return the file size
// 
// @param path         the path of the file
// 
// @return the size in bytes of the file (or -1 for error)
//
+ (long long) size :(const char *) path;

#ifdef WIN32
#endif
//
// Return the last modified time of a file
// 
// @param path         the path of the file
// 
// @return a (new) DDateTime object with the last modification date/time (or nil for error)
// 
+ (DDateTime *) modified :(const char *) path;

#ifdef WIN32
#endif
//
// Return the last accessed time of a file
// 
// @param path         the path of the file
// 
// @return a (new) DDateTime object with the last accessed date/time (or nil for error)
// 
+ (DDateTime *) accessed :(const char *) path;

#ifdef WIN32
#endif
@end
#endif
