#ifndef _DGRAPH_H
#define _DGRAPH_H

// This file was automatically generated by objch.

//==============================================================================
//
//            DGraph - the graph collection in the ofc-library
//
//               Copyright (C) 2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-11 16:59:46 $ $Revision: 1.3 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DList.h"

@class DGraphEdge;  // forward definition

//
// The graph node object implements a node in a graph. This node can store
// a different object so that it can be used as a collection. It can also
// store a label and attributes for graphviz visualation.
//

@interface DGraphNode : Object
{
@private
  DList         *_ingoing;    // the ingoing edges
  DList         *_outgoing;   // the outgoing edges
  DText         *_name;       // the name (set by DGraph)
  DText         *_label;      // the (graphviz) label
  DText         *_attributes; // the (graphviz) attributes
  id             _object;     // the stored object
  DGraphNode    *_prev;       // the previous node (during shortest path)
  double         _sum;        // the weight sum (during shortest path)
}



//// Constructors

//
// Initialise an empty graph node
//
// @return the object
//
- (DGraphNode *) init;

//
// Initialise a graph node with a label and object
//
// @param label    the label for the node
// @param object   the object
//
// @return the object
//
- (DGraphNode *) init :(const char *) label :(id) object;

//
// Initialise a graph node with a label, attributes and object
//
// @param label        the label for the node
// @param attributes   the (graphviz) attributes for the node
// @param object       the object
//
// @return the object
//
- (DGraphNode *) init :(const char *) label :(const char *) attributes :(id) object;


//// Copy related methods

//
// Do a shallow copy of the graph node (not implemented)
//
// @return the object
//
- shallowCopy;

//
// Deepen the copied object
//
// @return the object
//
- deepen;


//// Destructor

//
// (Shallow) Free the graph node (the stored object is NOT freed)
//
// @return the object
//
- shallowFree;

//
// Free the graph node (the stored object is freed)
//
// @return the object
//
- free;


//// Member methods

//
// Return the name of the graph node (used by DGraph)
// 
// @return the name of the node
// 
- (const char *) name;

//
// Set the name of the graph node (used by DGraph)
// 
// @param name    the name
// 
// @return the node object
// 
- (DGraphNode *) name :(const char *) name;

//
// Return the label of the graph node
//
// @return the label (or NULL)
//
- (const char *) label;

//
// Set the label of the graph node
//
// @param label   the label (or NULL)
// 
// Return the node object
// 
- (DGraphNode *) label :(const char *) label;

//
// Return the (graphviz) attributes of the node
// 
// @return the attributes (or NULL)
// 
- (const char *) attributes ;

//
// Set the (graphviz) attributes for the node
// 
// @param attributes    the attributes (or NULL)
// 
// @return the node object
// 
- (DGraphNode *) attributes :(const char *) attributes;

//
// Return the stored object in the node
// 
// @return the (stored) object (or nil)
// 
- (id) object;

//
// Set the object for the node
// 
// @param object    the object (or nil)
// 
// @return the previous stored object
// 
- (id) object :(id) object;


//// Path methods (used during shortest path determination)

//
// Reset the path
// 
// @return the node object
// 
- (DGraphNode *) path;

//
// Set the path 
// 
// @param prev    the previous node in the path
// @param sum     the sum of weights in the path
// 
// @return the node object
// 
- (DGraphNode *) path :(DGraphNode *) prev :(double) sum;

//
// Return the previous node in the path
// 
// @return the previous node (or nil)
// 
- (DGraphNode *) prev;

//
// Return the sum of weights in the path
// 
// @return the sum
// 
- (double) sum;


//// Edge methods

//
// Return the ingoing degree of the node
// 
// @return the number of ingoing edges
// 
- (unsigned long) ingoingDegree;

//
// Return the outgoing degree of the node
// 
// @return the number of output edges
// 
- (unsigned long) outgoingDegree;

//
// Return the degree of the node
// 
// @return the number of ingoing and outgoing edges
// 
- (unsigned long) degree;

//
// Add an ingoing edge
// 
// @param edge    the edge
// 
// @return the node object
// 
- (DGraphNode *) addIngoingEdge :(DGraphEdge *) edge;

//
// Remove an ingoing edge
// 
// @param edge    the edge
// 
// @return success
// 
- (BOOL) removeIngoingEdge :(DGraphEdge *) edge;

//
// Add an outgoing edge
// 
// @param edge    the edge
// 
// @return the node object
// 
- (DGraphNode *) addOutgoingEdge :(DGraphEdge *) edge;

//
// Remove an outgoing edge
// 
// @param edge    the edge
// 
// @return success
// 
- (BOOL) removeOutgoingEdge :(DGraphEdge *) edge;

//
// Replace edge with the copied edge (used during coping of graph)
// 
// @param orig     the original edge
// @param copy     the copied edge
// 
// @return the object
// 
- (DGraphNode *) replace :(DGraphEdge *) orig :(DGraphEdge *) copy;


//// Edge iterators

//
// Return the ingoing edge iterator
// 
// @return a (new) iterator on the ingoing edges
//
- (DListIterator *) ingoingEdges;

//
// Return the outgoing edge iterator
// 
// @return a (new) list iterator on the outgoing edges
// 
- (DListIterator *) outgoingEdges;

@end

//
// The graph edge object implements an edge in a graph. This edge can store
// another object, so that it can be used as a collection. It can also
// store a label and attributes for graphviz visualation.
//

@interface DGraphEdge : Object
{
@private
  DGraphNode    *_source;     // the source node
  DGraphNode    *_target;     // the target node
  DText         *_name;       // the name (set by DGraph)
  DText         *_label;      // the (graphviz) label 
  double         _weight;     // the weight of the edge (> 0.0)
  DText         *_attributes; // the (graphviz) attributes
  id             _object;     // the stored object
}

//// Constructors

//
// Initialise an empty graph edge
//
// @return the object
//
- (DGraphEdge *) init;

//
// Initialise a graph edge with a label, weight and object
//
// @param label    the label
// @param weight   the weight (> 0.0)
// @param object   the object
//
// @return the object
//
- (DGraphEdge *) init :(const char *) label :(double) weight :(id) object;

//
// Initialise a graph edge with a label, weight, attributes and object
//
// @param label        the label 
// @param attributes   the (graphviz) attributes
// @param weight       the weight (> 0.0)
// @param object       the object
//
// @return the object
//
- (DGraphEdge *) init :(const char *) label :(const char *) attributes :(double) weight :(id) object;


//// Copy related methods

//
// Do a shallow copy of the graph edge
//
// @return the object
//
- shallowCopy;

//
// Deepen a copied edge
//
// @return the object
//
- deepen;


//// Destructor

//
// Free the graph edge (the stored object is NOT freed)
//
// @return the object
//
- shallowFree;

//
// Free the graph edge (the stored object is freed)
//
// @return the object
//
- free;


//// Member methods

//
// Return the name of the edge (used by DGraph)
// 
// @return the name of the node
// 
- (const char *) name;

//
// Set the name of the edge (used by DGraph)
// 
// @param name    the name
// 
// @return the edge object
// 
- (DGraphEdge *) name :(const char *) name;

//
// Return the (graphviz) label of the graph edge
//
// @return the label (or NULL)
//
- (const char *) label;

//
// Set the (graphviz) label of the graph edge
//
// @param label   the label
// 
// Return the edge object
// 
- (DGraphEdge *) label :(const char *) label;

//
// Return the weight of the graph edge
//
// @return the weight
//
- (double) weight;

//
// Set the weight of the graph edge
//
// @param weight  the weight (> 0.0)
// 
// Return the edge object
// 
- (DGraphEdge *) weight :(double) weight;

//
// Return the (graphviz) attributes of the edge
// 
// @return the attributes (or NULL)
// 
- (const char *) attributes ;

//
// Set the (graphviz) attributes for the edge
// 
// @param attributes    the attributes
// 
// @return the edge object
// 
- (DGraphEdge *) attributes :(const char *) attributes;

//
// Return the stored object in the edge
// 
// @return the (stored) object (or nil)
// 
- (id) object;

//
// Set the object for the edge
// 
// @param object    the object
// 
// @return the previous stored object
// 
- (id) object :(id) object;


//// main methods

//
// Connect the source and target nodes by the edge
// 
// @param source    the source node (!= nil)
// @param target    the target node (!= nil)
// 
// @return success
// 
- (BOOL) connect :(DGraphNode *) source :(DGraphNode *) target;

//
// Disconnect the edge from the source and target node
// 
// @return the edge object
// 
- (DGraphEdge *) disconnect ;

//
// Return the source node
// 
// @return the source node
// 
- (DGraphNode *) source;

//
// Return the target node
// 
// @return the target node (or nil)
// 
- (DGraphNode *) target;

//
// Reverse the edge
// 
// @return the edge object
// 
- (DGraphEdge *) reverse;

//
// Replace node with the copied node (used during coping of graph)
// 
// @param orig     the original node
// @param copy     the copied node
// 
// @return the object
// 
- (DGraphEdge *) replace :(DGraphNode *) orig :(DGraphNode *) copy;

@end

  
#include "ofc/DTextWritable.h"
  
//
// The graph collection stores objects in directed graph with nodes and edges. 
// Both the nodes as the edges can store objects. The graph collection can also 
// be used as graph data structure. Using the toDot method, the graph structure 
// can be exported to a dot-file which can be visualized with the software of 
// @link(www.graphviz.org). IMPORTANT: as long as a node or edge object are part 
// of a graph, the node or edge object is owned by the graph object. Do not free 
// them outside the scope of the graph object.
// 
// @example
// #include <stdio.h>
// #include "ofc/DGraph.h"
// #include "ofc/DInt.h"
// #include "ofc/DFile.h"
// 
// int main(int argc, char *argv[])
// {
//   DGraph     *graph = [DGraph new];
//   DFile      *file  = [DFile  new];
// 
//   DList      *path;
// 
//   DGraphNode *node1,*node2,*node3;
//   DGraphEdge *edge1,*edge2,*edge3;
// 
//   DInt       *nr;
//   double      weight;
// 
//   [graph attributes :"rotate=90"];     // Set the graphviz attributes for the graph itself
// 
//   printf("Add the nodes to the graph.\n");
// 
//   nr = [DInt new]; [nr set :7];        // Create and add node to graph with label:start, attributes and object:7
//   node1 = [graph addNode :"start" :"shape=diamond" :nr];
// 
//   nr = [DInt new]; [nr set :3];        // Create and add node to graph with label:between, attributes and object:3
//   node2 = [graph addNode :"between" :"color=green" :nr];
// 
//   nr = [DInt new]; [nr set :9];        // Create and add node to graph with label:start, attributes and object:7
//   node3 = [graph addNode :"end" :"shape=circle" :nr];
// 
//   printf("Add the edges to the graph.\n");
// 
//   nr = [DInt new]; [nr set :-4];       // Create and add edge to graph from node1 to node2
//   edge1 = [graph addEdge :"edge1" :"color=blue" :1.0 :nr :node1 :node2];
//   
//   nr = [DInt new]; [nr set :-5];       // Create and add edge to graph from node2 to node3
//   edge2 = [graph addEdge :"edge2" :"style=dashed" :2.0 :nr :node2 :node3];
//   
//   nr = [DInt new]; [nr set :-3];       // Create and add edge to graph from node1 to node3
//   edge3 = [graph addEdge :"edge3" :"arrowhead=crow" :3.5 :nr :node1 :node3];
//  
//   path = [graph shortestPath :&weight :node1 :node3];
//   printf("Number of nodes in shortestpath:%ld with weight:%.1f\n", [path length], weight);
//   [path shallowFree];
// 
//   printf("Ingoing degree of node3:%lu\n",  [node3 ingoingDegree]);
//   printf("Outgoing degree of node2:%lu\n", [node2 outgoingDegree]);
//  
//   nr = [node2 object];
//   printf("Object of node2:%d\n", [nr get]);
// 
//   if ([file open :"example.dot" :"w"])
//   {
//     if ([graph toDot :file])
//       printf("Graph is succesfully exported to \"example.dot\".\n");
//     else
//       printf("Graph could not be exported to \"example.dot\".\n");
// 
//     [file close];
//   }
//   else
//     printf("Could not open the file \"example.dot\".\n");
// 
//   [graph free];                        // Cleanup
//   [file  free];
// 
//   return 0;
// }
//

@interface DGraph : Object
{
@private
  DList         *_nodes;       // the list with the graph nodes
  DList         *_edges;       // the list with the edges
  
  DText         *_attributes;  // the optional (graphviz) attributes
  
  long           _nodeCounter; // the node counter (for the node name)
  long           _edgeCounter; // the edge counter (for the edge name)
}

//// Constructors

//
// Initialise an empty graph 
//
// @return the object
//
- (DGraph *) init;

//
// Initialise a graph with attributes 
//
// @param attributes   the (graphviz) attributes
//
// @return the object
//
- (DGraph *) init :(const char *) attributes;


//// Copy related methods

//
// Do a shallow copy of the graph 
//
// @return the object
//
- shallowCopy;

//
// Deepen the copied graph
// 
// @return the object
// 
- deepen;


//// Destructor

//
// Free the graph (the stored objects are NOT freed)
//
// @return the object
//
- shallowFree;

//
// Free the graph  (the stored objects are freed)
//
// @return the object
//
- free;


//// Member methods

//
// Return the (graphviz) attributes of the graph
// 
// @return the attributes (or NULL)
// 
- (const char *) attributes ;

//
// Set the (graphviz) attributes for the graph
// 
// @param attributes    the attributes (or NULL)
// 
// @return the graph object
// 
- (DGraph *) attributes :(const char *) attributes;


//// Main methods

//
// Add a (new) node to the graph (graph is the owner of the node)
// 
// @param label       the label for the node (or NULL)
// @param attributes  the attributes for the node (or NULL)
// @param object      the object for the node (or nil)
// 
// @return the (new) node object
// 
- (DGraphNode *) addNode :(const char *) label :(const char *) attributes :(id) object;

//
// Add a (new) node to the graph (graph is the owner of the node)
// 
// @param node        the graph node
// 
// @return success
// 
- (BOOL) addNode :(DGraphNode *) node;

//
// Add a (new) edge to the graph (graph is the owner of the edge object)
// 
// @param label       the label for the edge (or NULL)
// @param attributes  the attributes for the edge (or NULL)
// @param weight      the weight for the edge
// @param object      the object for the edge (or nil)
// @param source      the source node
// @param target      the target node
// 
// @return the (new) edge object (or nil)
// 
- (DGraphEdge *) addEdge :(const char *) label :(const char *) attributes :(double) weight :(id) object :(DGraphNode *) source :(DGraphNode *) target;

//
// Add a (new) edge to the graph (graph is the owner of the edge object)
// 
// @param edge        the graph edge
// @param source      the source node 
// @param target      the target node
// 
// @return success
// 
- (BOOL) addEdge :(DGraphEdge *) edge :(DGraphNode *) source :(DGraphNode *) target;

//
// Reroute an existing edge in the graph
//  
// @param edge        the edge
// @param source      the new source for the edge
// @param target      the new target for the edge
// 
// @return success
// 
- (BOOL) reroute :(DGraphEdge *) edge :(DGraphNode *) source :(DGraphNode *) target;

//
// Determine the shortest path between two nodes; a (new) list with REFERENCES
// to the nodes in the shortest path are returned; IMPORTANT: free this list 
// with shallowFree
// 
// @param from      the start node
// @param to        the end node
// @param weight    the destination for the path weight (or NULL)
// 
// @return a (new) list with REFERENCES to the nodes in the shortes path (or nil)
//
- (DList *) shortestPath :(double *) weight :(DGraphNode *) from :(DGraphNode *) to;

//
// Check if the graph has a node
// 
// @param node      the graph node
// 
// @return has it ?
// 
- (BOOL) hasNode :(DGraphNode *) node;

//
// Check if the graph has an edge
// 
// @param edge      the graph edge
// 
// @return has it ?
// 
- (BOOL) hasEdge :(DGraphEdge *) edge;

//
// Remove an node from the graph (node must be unconnected)
// 
// @param node      the graph node
// 
// @return the object stored in the node (or nil)
// 
- (id) removeNode :(DGraphNode *) node;

//
// Remove an edge from the graph (auto disconnect)
// 
// @param edge      the edge node
// 
// @return the object stored in the edge (or nil)
// 
- (id) removeEdge :(DGraphEdge *) edge;


//// Iterators

//
// Return a (new) iterator on all the nodes in the graph
// 
// @return a (new) iterator
// 
- (DListIterator *) nodes;

//
// Return a (new) iterator on all the edges in the graph
// 
// @return a (new) list iterator
// 
- (DListIterator *) edges;


//// Export methods

//
// Export to DOT 
// 
// @param writer    the output object
// 
// @return success ?
// 
- (BOOL) toDot :(id <DTextWritable>) writer;

@end
#endif
