#ifndef _DGRAPHICSCREEN_H
#define _DGRAPHICSCREEN_H

// This file was automatically generated by objch.

//==============================================================================
//
//     DGraphicScreen - the graphic screen wrapper class in the ofc-library
//
//               Copyright (C) 2007  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-26 05:50:30 $ $Revision: 1.18 $
//
//==============================================================================
#ifdef HAVE_DGRAPHICSCREEN
#endif

#include "ofc/config.h"

#include "ofc/DColor.h"

#include "ofc/DScreenHandler.h"
#include "ofc/DDrawable.h"

#include "ofc/DFont.h"
#include "ofc/DSimpleFont.h"
#include "ofc/DImage.h"


#if defined(HAVE_SDL)
# define HAVE_DGRAPHICSCREEN (1)
#endif



#ifdef HAVE_DGRAPHICSCREEN

//
// The DGraphicDrawable class implements a class with the shared methods between
// the graphic screen class and the graphic surface class. It contains the methods
// for drawing, clipping, color.. The x and y positions start from zero, so the
// screen size is 0..maxX and 0..maxY (inclusive).
// ToDo: drawing boxes, circles..
// 

@interface DGraphicDrawable : Object <DDrawable>
{
@protected
  unsigned             _maxX;        // the right x-position of the drawable
  unsigned             _maxY;        // the bottom y-position of the drawable
  
  DColor              *_fgc;         // the current foreground color
  DColor              *_bgc;         // the current background color
  id <DFont>           _font;        // the current font
  DSimpleFont         *_basicFont;   // the basic font
  
  void                *_surface;     // the surface of the drawable
  
@private
  BOOL                 _drawing;     // is drawing active ?
  
  unsigned             _clipMinX;    // the left x-position for the clip area
  unsigned             _clipMaxX;    // the right x-position for the clip area
  unsigned             _clipMinY;    // the top y-position for the clip area
  unsigned             _clipMaxY;    // the bottom y-position for the clip area
  unsigned             _cursorX;     // the x-position of the cursor
  unsigned             _cursorY;     // the y-position of the cursor
  unsigned             _nextY;       // the next y-position after writing text
  
  int                  _lineType;    // the last line type
  int                  _dashLength;  // the length of a dash
  int                  _bezierSteps; // the number of iteration steps for a bezier curve (def. 15)
}

#endif
#ifdef HAVE_DGRAPHICSCREEN
//
// Initialise an graphic drawable object
//
// @return the object
//
- (DGraphicDrawable *) init;


//// Copy related methods

//
// Do a shallow copy of the graphic drawable object
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the graphic screen
// 
// @return the object
// 
- free;


//// Member methods

//
// Check if the graphic drawable object is open
// 
// @return is it ?
//
- (BOOL) isOpen;

//
// Return the number of bezier iteration steps (def. 15)
// 
// @return the number of steps
// 
- (int) bezierSteps;

//
// Set the number of bezier iteration steps
// 
// @param steps    the number of bezier steps (>1)
// 
// @return the object
// 
- (DGraphicDrawable *) bezierSteps :(int) steps;

//
// Return the current line type
// 
// @return the line type (DDW_LINE_SOLID,DDW_LINE_DASHED,DDW_LINE_DOTTED)
// 
- (int) lineType;

//
// Set the current line type
// 
// @param type     the line type (DDW_LINE_SOLID,DDW_LINE_DASHED,DDW_LINE_DOTTED)
// 
// @return the object
// 
- (DGraphicDrawable *) lineType :(int) type;

//
// Return the length of a dash (def. 10)
// 
// @return the dash length
// 
- (int) dashLength;

//
// Set the length of a dash
// 
// @param length   the dash length
// 
// @return the object
// 
- (DGraphicDrawable *) dashLength :(int) length;


//// Cursor methods

//
// Return the x position of the cursor
// 
// @return the x position
// 
- (unsigned) cursorX;

//
// Return the y position of the cursor
// 
// @return the y position
//
- (unsigned) cursorY;

//
// Set the cursor to a position
// 
// @param x     the x-position
// @param y     the y-position
// 
// @return success
// 
- (BOOL) cursor :(unsigned) x :(unsigned) y;

//
// Set the visibility of the cursor
// 
// @param state   the cursor state (DDW_CURSOR..)
// 
// @return the previous state of the cursor
// 
- (int) cursor :(int) state;

//
// Set the visibility of the mouse pointer
// 
// @param state   the pointer state (DDW_POINTER..)
// 
// @return the previous state of the pointer
// 
- (int) pointer :(int) state;

//
// Return the last error
// 
// @return the last error (0)
// 
- (int) error;

//
// Tell the current position in the writeable
//
// @return the current position
//
- (unsigned long) tell;

//
// Move the current position to a location
//
// @param offset   the offset from the origin (in positions)
// @param origin   the origin for the offset (0=begin, 1=current, 2=end)
//
// @return success
//
- (BOOL) seek :(unsigned long) offset :(int) origin;

//
// Skip a number of positions
//
// @param offset   the number of positions to skip
//
// @return success
//
- (BOOL) skip :(unsigned long) offset;


//// Font methods

//
// Set the current font
//
// @param font     the (reference to the) font (or nil for none)
//
// @return the (reference to the) previous font
//
- (id <DFont>) font :(id <DFont>) font;

//
// Get the current font
//
// @return the (reference to the) current font (or nil)
//
- (id <DFont>) font;


//// Surface methods

//
// Return the maximum x-position
// 
// @return the maximum x-position
//
- (unsigned) maxX;

//
// Return the height
// 
// @return the height
// 
- (unsigned) height;

//
// Return the maximum y-position
// 
// @return the maximum y-position
// 
- (unsigned) maxY;

//
// Return the width
// 
// @return the width
// 
- (unsigned) width;

//
// Set the dimensions of the drawable
// 
// @param width  the width (> 0)
// @param height the height (> 0)
// 
// @return the object
// 
- (DGraphicDrawable *) drawable :(unsigned) width :(unsigned) height;

//
// Check if a cursor position is valid (within the drawable)
// 
// @param x     the x-position
// @param y     the y-position
// 
// @return is it ?
// 
- (BOOL) isValid :(unsigned) x :(unsigned) y;


//// Clipping methods

//
// Set a clipping area
// 
// @param minX    the left x-position of the clip area
// @param maxX    the right x-position of the clip area
// @param minY    the top y-position of the clip area
// @param maxY    the bottom y-position of the clip area
// 
// @return success
// 
- (BOOL) clip :(unsigned) minX :(unsigned) minY :(unsigned) maxX :(unsigned) maxY;

//
// Reset the clipping area to the full surface
// 
// @return success
// 
- (BOOL) clip;

//
// Return the top position of the clip area
// 
// @return the top y-position
// 
- (unsigned) clipMinY;

//
// Return the bottom position of the clip area
// 
// @return the bottom y-position
// 
- (unsigned) clipMaxY;

//
// Return the left position of the clip area
// 
// @return the left x-position
// 
- (unsigned) clipMinX;

//
// Return the right position of the clip area
// 
// @return the right x-position
// 
- (unsigned) clipMaxX;


//// Color methods

//
// Set the fore- and background color
// 
// @param fgc     the foreground color
// @param bgc     the background color
// 
// @return success
// 
- (BOOL) color :(DColor *) fgc :(DColor *) bgc;

//
// Set the foreground color
// 
// @param fgc     the foreground color
// 
// @return success
// 
- (BOOL) foregroundColor :(DColor *) fgc;

//
// Get the foreground color
// 
// @return a reference to the foreground color
// 
- (DColor *) foregroundColor;

//
// Set the background color
// 
// @param bgc     the background color
// 
// @return success
// 
- (BOOL) backgroundColor :(DColor *) bgc;

//
// Get the background color
// 
// @return a reference to the background color
// 
- (DColor *) backgroundColor;


//// Drawing control methods

//
// Start drawing on the screen
// 
// @return success
// 
- (BOOL) startDrawing;

//
// Start drawing in a clipped area on the screen
// 
// @param minX      the left x-position of the clipping area
// @param maxX      the right x-position of the clipping area
// @param minY      the top y-position of the clipping area
// @param maxY      the bottom y-position of the clipping area
// 
// @return success
//
- (BOOL) startDrawing :(unsigned) minX :(unsigned) minY :(unsigned) maxX :(unsigned) maxY;

//
// Stop drawing on the screen
// 
// @return the screen object
// 
- (DGraphicDrawable *) stopDrawing;

//
// Check if the screen is in drawing mode
// 
// @return is it ?
// 
- (BOOL) isDrawing;


//// Drawing methods

//
// Clear the drawable (in background color)
// 
// @return success
// 
- (BOOL) clear;

//
// Write a text string on the screen on the current cursor position
// 
// @param text    the text string
// 
// @return success
// 
- (BOOL) writeText :(const char *) text;

//
// Write a character on the screen on the current cursor position
// 
// @param ch      the character
// 
// @return success
// 
- (BOOL) writeChar :(char) ch;

//
// Write a text line on the screen on the current cursor position
// 
// @param text    the text string
// 
// @return success
// 
- (BOOL) writeLine :(const char *) text;

//
// Write a text string on the screen
// 
// @param startX  the start x position
// @param startY  the start y position
// @param text    the text string
// 
// @return success
// 
- (BOOL) writeText :(unsigned) startX :(unsigned) startY :(const char *) text;

//
// Write a character on the screen
// 
// @param startX  the start x position
// @param startY  the start y position
//
// @param ch      the character
// 
// @return success
// 
- (BOOL) writeChar :(unsigned) startX :(unsigned) startY :(char) ch;


//// Image methods

//
// Write an image to the drawable
// 
// @param startX   the start x position
// @param startY   the start y position
// @param image    the source image object
// 
// @return success
// 
- (BOOL) writeImage :(unsigned) startX :(unsigned) startY :(id <DImage>) image;

//
// Read an image from the drawable
// 
// @param startX     the start x position
// @param startY     the start y position
// @param image      the destination image object
// 
// @return success
// 
- (BOOL) readImage :(unsigned) startX :(unsigned) startY :(id <DImage>) image;

//
// Draw a (horizontal) line on the drawable
//
// @param startX    the x position for the start of the line
// @param endX      the x position for the end of the line
// @param startY    the y position for the line
// @param lineType  the type of the line DDW_LINE_...
//
// @return success
//
- (BOOL) drawHLine :(unsigned) startX :(unsigned) startY :(unsigned) endX :(int) lineType;

//
// Draw a horizontal line with the previous line width and type settings
//
// @param startX    the x position for the start of the line
// @param endX      the x position for the end of the line
// @param startY    the y position for the line
//
// @return success
//
- (BOOL) drawHLine :(unsigned) startX :(unsigned) startY :(unsigned) endX ;

//
// Draw a (horizontal) line on the drawable starting from the current position and
// with the previous line width and type settings
//
// @param endX      the x position for the end of the line
//
// @return success
//
- (BOOL) drawHLine : (unsigned) endX ;

//
// Draw a vertical line on the screen
// 
// @param startX      the x position for the line
// @param startY      the y position for the line
// @param endY        the y position for the end of the line
// @param lineType    the type of the line (DDW_LINE_...)
// 
// @return success
// 
- (BOOL) drawVLine :(unsigned) startX :(unsigned) startY :(unsigned) endY :(int) lineType;

//
// Draw a vertical line with the current line type and width settings
// 
// @param startX      the x position for the line
// @param startY      the y position for the line
// @param endY        the y position for the end of the line
// 
// @return success
// 
- (BOOL) drawVLine :(unsigned) startX :(unsigned) startY :(unsigned) endY;

//
// Draw a (vertical) line on the drawable starting from the current position and
// with the current line width and type settings
//
// @param endY      the y position for the end of the line
//
// @return success
//
- (BOOL) drawVLine :(unsigned) endY;

//
// Draw a line on the screen
//
// @param startX      the x position for the start of the line 
// @param endX        the x position for the end of the line
// @param startY      the y position for the start of the line
// @param endY        the y position for the end of the line
// @param lineType    the type of the line (DDW_LINE_...)
// 
// @return success
// 
- (BOOL) drawLine :(unsigned) startX :(unsigned) startY :(unsigned) endX :(unsigned) endY :(int) lineType;

//
// Draw a line with the current line type and width settings
//
// @param startX  the x position for the start of the line 
// @param endX    the x position for the end of the line
// @param startY  the y position for the start of the line
// @param endY    the y position for the end of the line
// 
// @return success
// 
- (BOOL) drawLine :(unsigned) startX :(unsigned) startY :(unsigned) endX :(unsigned) endY;

//
// Draw a line on the drawable starting from the current position and
// with the current line width and type settings
//
// @param endX      the x-position for the end of the line
// @param endY      the y-position for the end of the line
//
// @return success
//
- (BOOL) drawLine :(unsigned) endX :(unsigned) endY;

//
// Draw a bezier line on the screen
//
// @param startX      the x position for the start of the line 
// @param startY      the y position for the start of the line
// @param strrcX      the x position for the start direction point
// @param strrcY      the y position for the start direction point
// @param endrcX      the x position for the end direction point
// @param endrcY      the y position for the end direction point
// @param endX        the x position for the end of the line
// @param endY        the y position for the end of the line
// 
// @return success
// 
- (BOOL) drawBezierLine :(unsigned) startX :(unsigned) startY 
                        :(unsigned) strrcX :(unsigned) strrcY
                        :(unsigned) endrcX :(unsigned) endrcY
                        :(unsigned) endX   :(unsigned) endY;

//
// Draw a point on the drawable
// 
// @param startX   the x-position
// @param startY   the y-position
// 
// @return success
// 
- (BOOL) drawPoint :(unsigned) startX :(unsigned) startY;

//
// Draw a point on the current position
// 
// @return success
// 
- (BOOL) drawPoint;

//
// Blit another graphic drawable on this graphic drawable (without alpha blending)
// 
// @param startX    the start x-position where the info is placed on this drawable
// @param startY    the start y-position where the info is placed on this drawable
// @param other     the other drawable
// @param oStartX   the start x-position on the other drawable
// @param oStartY   the start y-position on the other drawable
// @param oEndX     the end x-position on the other drawable
// @param oEndY     the end y-position on the other drawable
// 
// @return success
// 
- (BOOL) blit :(unsigned) startX :(unsigned) startY 
              :(DGraphicDrawable *) other :(unsigned) oStartX :(unsigned) oStartY :(unsigned) oEndX :(unsigned) oEndY;


//// Private methods

@end
#endif

#ifdef HAVE_DGRAPHICSCREEN


//
// The DGraphicScreen class implements a class to use the graphic screen. It
// is a wrapper for the sdl library. For obvious reasons there can be only
// one graphic screen object in any application.
// 
// @example
// #include <stdio.h>
// #include "ofc/DGraphicScreen.h"
// #include "ofc/DTrueTypeFont.h"
// #include "ofc/DPNGImage.h"
// #include "ofc/DFile.h"
// 
// @interface MyScreen : DGraphicScreen <DScreenHandler>  // Class for the screen and its events
// {
//   DColor          *_fgc;
//   DColor          *_bgc;
//   DTrueTypeFont   *_font;
//   DGraphicSurface *_image;
// }
// - (MyScreen *) init;
// - (BOOL) open;
// - (void) draw :(const char *) key;
// - (BOOL) keyPress :(DKey *) key :(int) state;      // Callbacks
// - (BOOL) mouseMove :(unsigned) x :(unsigned) y;
// - (BOOL) mouseButtonPress :(DKey *) button :(int) state;
// - (BOOL) screenResize :(unsigned) maxX :(unsigned) maxY;
// - (BOOL) screenFocus :(BOOL) focus;
// - (BOOL) screenRedraw :(unsigned) minX :(unsigned) maxX :(unsigned) minY :(unsigned) maxY;
// @end
// 
// @implementation MyScreen
// 
// - (MyScreen *) init
// {
//   [super init];
//   
//   _fgc  = [DColor new];
//   _bgc  = [DColor new];
//   _font = [DTrueTypeFont new];
//   _image = [DGraphicSurface new];
// 
//   return self;
// }
// 
// - (BOOL) open                          // Open the screen with ..
// {
//   BOOL ok = NO;
// 
//   [_fgc textColor :DCLR_BLACK];       // .. foreground and background color
//   [_bgc textColor :DCLR_WHITE];
//   
//   ok = [self open :600 :400 :0 :_fgc :_bgc];
// 
//   if (ok)
//   {
//     [self screenHandler :self];        // Object is handling its events itself
//   }
// 
//   if (ok)
//   {
//     ok = [_font open :"../test/arial.ttf"]; // Open the arial font
// 
//     if (ok)
//     {
//       [_font size :"a" :15 :15];       // Size the font
// 
//       [self font :_font];              // Make it the default font
//     }
// 
//     if (ok)
//     {
//       DFile *file    = [DFile new];
//       DPNGImage *png = [DPNGImage new];
// 
//       if ([file open :"ofc.png" :"rb"]) // Open the png image file
//       {
//         if ([png open :file])          // Give the file to the png image
//         {                              // Open a graphic surface for the image
//           if ([_image open :[png width] :[png height] :_bgc :_bgc])
//           {
//             [_image startDrawing];
//             [_image writeImage :0 :0 :png]; // Write the image to the surface
//             [_image stopDrawing];
//           }
//           [png close];
//         }
//         [file close];
//        }
//     }
//   }
//   
//   return ok;
// }
// 
// - (void) draw :(const char *) key      // Screen drawing
// {
//   int maxX = [self maxX];              // Dimensions of screen
//   int maxY = [self maxY];
// 
//   char hello[] = "Hello World";
//   char quit[]  = "Press q to quit";
// 
//   unsigned width,height;
// 
//   [self startDrawing];                 // Start drawing
//   [self clear];
//   [self drawHLine :10 :10 :maxX-10];      // Draw a some lines
//   [self drawHLine :maxX-10 :maxY-10 :10];
//                                        // Write some text
//   [_font stringSize :hello :&width :&height];
// 
//   [self cursor :((maxX - width)/2) :((maxY-40)/2)];
//   [self writeText :hello];
// 
//   [_font stringSize :quit :&width :&height];
// 
//   [self cursor :((maxX - width)/2) :((maxY+40)/2)];
//   [self writeText :quit];
// 
//   if (key != NULL)
//   {
//     [self cursor :30 :maxY-30];
//     [self writeText :"Key pressed:"];
//     [self writeText :key];
//   }
//                                        // Blit the image surface on the screen
//   [self blit :10 :15 :_image :0 :0 :[_image maxX] :[_image maxY]];
// 
//   [self stopDrawing];
// }
//                                        // Callback for pressed key
// - (BOOL) keyPress :(DKey *) key :(int) state
// {
//   if ([key key] != 'q')
//   {
//     DText *info = [key toText];
//     
//     [self draw :[info cstring]];
// 
//     [info free];
//   }
//   
//   return ([key key] != 'q');
// }
//                                        // Callback for moved mouse
// - (BOOL) mouseMove :(unsigned) x :(unsigned) y
// {
//   return YES;
// }
//                                        // Callback for pressed mouse button
// - (BOOL) mouseButtonPress :(DKey *) button :(int) state
// {
//   return YES;
// }
//                                        // Callback for screen resize
// - (BOOL) screenResize :(unsigned) maxX :(unsigned) maxY
// {
//   [self draw :NULL];
// 
//   return YES;
// }
//                                        // Callback for lost and gained screen focus
// - (BOOL) screenFocus :(BOOL) focus
// {
//   [self draw :NULL];
// 
//   return YES;
// }
//                                        // Callback for screen redraw
// - (BOOL) screenRedraw :(unsigned) minX :(unsigned) maxX :(unsigned) minY :(unsigned) maxY
// {
//   [self draw :NULL];
// 
//   return YES;
// }
// 
// @end
// 
// int main(int argc, char *argv[])
// {
//   MyScreen *screen = [MyScreen new];
// 
//   if ([screen open])                   // Open the screen
//   {
//     [screen draw :NULL];               // Draw the screen
// 
//     [screen waitEvents];               // Wait for and process events
//   }
//   else
//     printf("Could not open the screen..\n");
// 
//   [screen free];                       // Cleanup
// 
//   return 0;
// }
// 

@interface DGraphicScreen : DGraphicDrawable
{
@private
  id <DScreenHandler>  _handler;     // the screen event handler
  DKey                *_key;         // the key
  unsigned             _mask;        // the current key mask
  int                  _bbp;         // the bits per pixel or 0 for any
}

#endif
#ifdef HAVE_DGRAPHICSCREEN

//// Constructors

//
// Initialise to an unopened graphic screen object
//
// @return the object
//
- (DGraphicScreen *) init;

//
// Initialise and open a graphic screen object with a fore- and background color
// 
// @param width   the width of the screen
// @param height  the height of the screen
// @param bbp     the depth of the screen (0 = current depth)
// @param fgc     the foreground color
// @param bgc     the background color
//
// @return the object
//
- (DGraphicScreen *) init :(unsigned) width :(unsigned) height :(unsigned) bbp :(DColor *) fgc :(DColor *) bgc;


//// Copy related methods

//
// Do a shallow copy of the graphic screen object (not implemented)
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the graphic screen
// 
// @return the object
// 
- free;


//// Screen status methods


//// Event handler methods

//
// Set the screen event handler
// 
// @param handler   the (reference to the) screen handler
// 
// @return the previous screen handler
// 
- (id <DScreenHandler>) screenHandler :(id <DScreenHandler>) handler;

//
// Return the current screen event handler
// 
// @return the (reference to the) screen handler
// 
- (id <DScreenHandler>) screenHandler;


//// Screen control methods

//
// Open the graphic screen (only one instance is opened)
//
// @param width   the width of the screen
// @param height  the height of the screen
// @param bbp     the depth of the screen (0 = current depth)
// @param fgc     the foreground color
// @param bgc     the background color
// 
// @return success
// 
- (BOOL) open :(unsigned) width :(unsigned) height :(unsigned) bbp :(DColor *) fgc :(DColor *) bgc;

// 
// Close the graphic screen
// 
// @return the object
// 
- (DGraphicScreen *) close;


//// Screen output methods

//
// Beep the terminal
//
// @return the object
//
- (DGraphicScreen *) beep ;

//
// Set the visibility of the mouse pointer
// 
// @param state   the pointer state (DDW_POINTER..)
// 
// @return the previous state of the pointer
// 
- (int) pointer :(int) state;


//// Drawing control methods

//
// Stop drawing on the screen
// 
// @return the screen object
// 
- (DGraphicScreen *) stopDrawing;


//// Event processing

//
// Wait (and process) events
// 
// @return the number of processed events
// 
- (unsigned) waitEvents;

//
// Process any pending events
//
// @return the number of processed events
// 
- (unsigned) processEvents;

//
// Clear any pending events
//
// @return the object
// 
- (DGraphicScreen *) clearEvents;

@end
#endif

#ifdef HAVE_DGRAPHICSCREEN


//
// The DGraphicSurface class implements a class for off-screen drawing to a graphic
// surface. This surface can then be blit to the graphic screen.
// 

@interface DGraphicSurface : DGraphicDrawable
{
}

#endif
#ifdef HAVE_DGRAPHICSCREEN

//// Constructors

//
// Initialise to an graphic surface object
//
// @return the object
//
- (DGraphicSurface *) init;

//
// Initialise and open a graphic surface object 
// 
// @param width   the width of the screen
// @param height  the height of the screen
// @param fgc     the (default) foreground color
// @param bgc     the (default) background color
//
// @return the object
//
- (DGraphicSurface *) init :(unsigned) width :(unsigned) height :(DColor *) fgc :(DColor *) bgc;


//// Copy related methods

//
// Do a shallow copy of the graphic surface object (not implemented)
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the graphic surface
// 
// @return the object
// 
- free;


//// Surface control methods

//
// Open a graphic screen surface (with the same depth as the screen)
//
// @param width     the width of the surface
// @param height    the height of the surface
// @param fgc       the (default) foreground color
// @param bgc       the (default) background color
// 
// @return success
// 
- (BOOL) open :(unsigned) width :(unsigned) height :(DColor *) fgc :(DColor *) bgc;

//
// Close the surface
// 
// @return the object
// 
- (DGraphicSurface *) close;

@end
#endif
#endif
