#ifndef _DIMAGEHEADER_H
#define _DIMAGEHEADER_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DImageHeader - the Image Header class in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-02 05:58:14 $ $Revision: 1.9 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DDataReadable.h"

#include "ofc/DData.h"
#include "ofc/DText.h"

// DImageHeaderType

typedef enum
{
  DIM_UNKN = -1,  // unknown image format
  DIM_JPEG,       // jpeg image format
  DIM_GIF,        // gif image format
  DIM_PNG,        // png image format
  DIM_BMP,        // bmp image format
  DIM_PCX,        // pcx image format
  DIM_IFF,        // iff image format
  DIM_RAS,        // ras image format
  DIM_PBM,        // pbm image format
  DIM_PGM,        // pgm image format
  DIM_PPM,        // ppm image format
  DIM_PSD,        // psd image format
  DIM_SWF,        // swf image format
  DIM_TIFF        // tiff image format
} DImageHeaderType;

//
// The DImageHeader class implements a number of methods for determining the
// type of an image file. Due to the fact that only small tests are performed
// on the image files, the determination is not full proof. In the future this 
// will become a factory for image objects.
//
// @example
// #include <stdio.h>
// #include "ofc/DImageHeader.h"
// #include "ofc/DFile.h"
// 
// int main(int argc, char *argv[])
// {
//   DImageHeader *hdr  = [DImageHeader new];
//   DFile        *file = [DFile new];
// 
//   if ([file open :"example.jpg" :"r"])
//   {
//     if ([hdr inspect :file])
//       printf("File type:%d Mime Type:%s File Extension:%s\n", [hdr type], [hdr mime_type], [hdr extension]);
//     else
//       printf("No image type could be detected for file \"example.jpg\".\n");
// 
//     [file close];
//   }
//   else
//     printf("File \"example.jpg\" could not be opened for inspection.\n");
// 
//   [file free];                         // Cleanup
//   [hdr  free];
// 
//   return 0;
// }
// 

@interface DImageHeader : Object
{
@private
  DImageHeaderType _type;    // the image type (DIM_UNKN,DIM_JPEG,DIM_GIF,DIM_PNG,DIM_BMP,DIM_PCX,DIM_IFF,DIM_RAS,DIM_PBM,DIM_PGM,DIM_PPM,DIM_PSD,DIM_SWF)
}


//// Constructors

//
// Initialise an empty image header object
//
// @return the instance
//
- (DImageHeader *) init;

//
// Initialise with a data readable
//
// @param file     the file
//
// @return the instance
//
- (DImageHeader *) init :(id <DDataReadable>) file;


//// Image info methods

//
// Return the type of the image (or DIM_UNKN for unknown)
//
// @return the type
//
- (DImageHeaderType) type;

//
// Return the extension for the type of image
//
// @return the extension (or NULL for unknown type)
//
- (const char *) extension;

//
// Return the mime type for the type of image
//
// @return the mime type (or NULL for unknown type)
//
- (const char *) mime_type;


//// Main method

//
// Inspect the image file
//
// @param file     the file to be inspected
//
// @return success
//
- (BOOL) inspect :(id <DDataReadable>) file;

@end
#endif
