#ifndef _DJPEGIMAGE_H
#define _DJPEGIMAGE_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DJPEGImage - the JPEG image class in the ofc-library
//
//               Copyright (C) 2007  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-20 17:16:29 $ $Revision: 1.4 $
//
//==============================================================================
#ifdef HAVE_DJPEGIMAGE
#endif

#include "ofc/config.h"

#include "ofc/DImage.h"
#include "ofc/DDataReadable.h"
#include "ofc/DDataWritable.h"

#if defined(HAVE_LIBJPEG) && defined(HAVE_JPEGLIB_H)
#define HAVE_DJPEGIMAGE 1
#endif

#ifdef HAVE_DJPEGIMAGE

//
// The DJPEGImage class implements methods for reading and writing JPEG images.
// 
// @example
// #include <stdio.h>
// #include "ofc/DJPEGImage.h"
// #include "ofc/DFile.h"
// 
// int main(int argc, char *argv[])
// {
//   DJPEGImage *image = [DJPEGImage new];
//   DFile      *file  = [DFile new];
// 
//   char       name[] = "example.jpg";
// 
//   if ([file open :name :"rb"])
//   {
//     if ([DJPEGImage isImage :file])
//     {
//       if ([image open :file])
//       {
//         printf("\"%s\" is a JPEG image with width:%d height:%d BytesPerPixel:%d Images:%d\n",
//             name, [image width], [image height], [image bytesPerPixel], [image images]);
//         
//         [image close];
//       }
//       else
//         printf("Could not open image \"%s\".\n", name);
//     }
//     else
//       printf("\"%s\" is not a JPEG image.\n", name);
// 
//     [file close];
//   }
//   else
//     printf("Could not find \"%s\":%d\n", name, [file error]);
// 
//   [image free];                        // Cleanup
//   [file  free];
//   
//   return 0;
// }
// 

@interface DJPEGImage : Object <DImage>
{
@private
  void                *_jinfo;         // the jpeg info
  void                *_jerr;          // the jpeg error
  unsigned             _height;        // the height of the image
  unsigned             _width;         // the width of the image
  unsigned             _bpp;           // the bytes per pixel
  BOOL                 _reading;       // reading or writing an image ?
  BOOL                 _error;         // is an error active ?
}

#endif
#ifdef HAVE_DJPEGIMAGE
#ifdef _PRIVATE_

//// DJPEG error callback functions

//
// Do error exit
// 
// @param jinfo    the jpeg info
// 
//
// Output message
// 
// @param jinfo    the jpeg info
// 
//

//// DJPEG reading callback functions

//
// Initialise the source buffer for every image
// 
// @param jinfo    the jpeg info
// 
//
// Fill the input buffer
// 
// @param jinfo    the jpeg info
// 
// @return success
// 
//
// Skip input data
// 
// @param jinfo    the jpeg info
// @param nbyte    the number of bytes to skip
// 
//
// Terminate the input source
// 
// @param jinfo    the jpeg info
// 

//// DJPEG writing callback functions

//
// Initialise the destination buffer
// 
// @param jinfo      the jpeg info
// 
//
// Empty the output buffer
// 
// @param jinfo    the jpeg info
// 
// @return completion
// 
//
// Terminate destination buffer
// 
// @param jinfo    the jpeg info
// 
#endif
//
// Initialise a JPEG image object
//
// @return the object
//
- (DJPEGImage *) init;


//// Copy related methods

//
// Do a shallow copy of a JPEG image (not implemented)
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the image object
// 
// @return the object
// 
- free;


//// Image members

//
// Return the height of the image
// 
// @return the height
// 
- (unsigned) height;

//
// Return the width of the image
// 
// @return the width
// 
- (unsigned) width;

//
// Return the bytes per pixel (3: RGB or 4: RGBA)
// 
// @return the number of bytes
// 
- (unsigned) bytesPerPixel;

//
// Return the number of images in the image (jpeg: always 1)
// 
// @return the number of images
// 
- (unsigned) images;


//// Image methods

//
// Check if an image is an JPEG image
// 
// @param source        the source of the image
// 
// @return is it ?
// 
+ (BOOL) isImage :(id <DDataReadable>) source;

//
// Open the JPEG image object for reading
// 
// @param source        the source with the image
//
// @return success
//
- (BOOL) open :(id <DDataReadable>) source;

//
// Open the JPEG image object for writing (RGB = 3 bytesPerPixel, no alpha)
// 
// @param destination        the file for the image
// @param width              the width of the image
// @param height             the height of the image
// @param quality            the quality [0..100]
//
// @return succes
//
- (BOOL) open :(id <DDataWritable>) destination :(unsigned) width :(unsigned) height :(int) quality;

//
// Get the next row of image data from the jpgeg file (format: RGB)
// 
// @param data          the image data for the next line (size = _width * _bpp)
//
// @return the number of pixels in the row (or 0 for error)
//
- (unsigned) readRow :(unsigned char *) data;

//
// Set the next row of image data in the jpeg file (format: RGB)
// 
// @param data          the image data for the next line (size = _width * _bpp)
// @param width         the number of pixels in the row
//
// @return success
//
- (BOOL) writeRow :(unsigned char *) data :(unsigned) width;

//
// Close the image object
// 
// @return success
//
- (BOOL) close;

@end
#endif
#endif
