#ifndef _DKEY_H
#define _DKEY_H

// This file was automatically generated by objch.

//==============================================================================
//
//            DKey - the (keyboard) key class in the ofc-library
//
//               Copyright (C) 2004  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-05 16:29:04 $ $Revision: 1.5 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DText.h"

#include "ofc/DParsable.h"
#include "ofc/DTextable.h"

//
// The key codes
// 
#define DKEY_NULL            (0)
#define DKEY_BACKSPACE       (8)
#define DKEY_ENTER           (13)   // the enter key
#define DKEY_ESCAPE          (27)
#define DKEY_DELETE          (127)

// Function keys
#define DKEY_F1              (282)  // Function key 1
#define DKEY_F2              (283)  // Function key 2
#define DKEY_F3              (284)  // Function key 3
#define DKEY_F4              (285)  // Function key 4
#define DKEY_F5              (286)  // Function key 5
#define DKEY_F6              (287)  // Function key 6
#define DKEY_F7              (288)  // Function key 7
#define DKEY_F8              (289)  // Function key 8
#define DKEY_F9              (290)  // Function key 9
#define DKEY_F10             (291)  // Function key 10
#define DKEY_F11             (292)  // Function key 11
#define DKEY_F12             (293)  // Function key 12
#define DKEY_F13             (294)  // Function key 13
#define DKEY_F14             (295)  // Function key 14
#define DKEY_F15             (296)  // Function key 15
#define DKEY_F16             (297)  // Function key 16
#define DKEY_F17             (298)  // Function key 17
#define DKEY_F18             (299)  // Function key 18
#define DKEY_F19             (300)  // Function key 19
#define DKEY_F20             (301)  // Function key 20

// Modifier keys
#define DKEY_NUMLOCK         (302)  // Numlock key
#define DKEY_CAPSLOCK        (303)  // Capslock key
#define DKEY_SCROLLLOCK      (304)  // Scroll key
#define DKEY_SHIFT           (305)  // Shift key
#define DKEY_CTRL            (306)  // Control key
#define DKEY_ALT             (307)  // Alt key
#define DKEY_KP              (308)  // Keypad key
#define DKEY_MOUSE           (309)  // 'Mouse' key
#define DKEY_MIDDLE          (310)  // Middle key

// Cursor movement
#define DKEY_UP              (273)  // Up key
#define DKEY_DOWN            (274)  // Down key
#define DKEY_RIGHT           (275)  // Right key
#define DKEY_LEFT            (276)  // Left key
#define DKEY_INSERT          (277)  // Insert key
#define DKEY_HOME            (278)  // Home key
#define DKEY_END             (279)  // End key
#define DKEY_PAGEUP          (280)  // Page up key
#define DKEY_PAGEDOWN        (281)  // Page down key

// Modifiers
#define DKEY_MOD_SHIFT       (0x1000)  // the shift modifier
#define DKEY_MOD_CTRL        (0x2000)  // the control modifier
#define DKEY_MOD_ALT         (0x4000)  // the alt modifier
#define DKEY_MOD_KP          (0x8000)  // the keypad modifier


// Masks
#define DKEY_KEY_MASK        (0x0FFF)  // the key mask
#define DKEY_MOD_MASK        (DKEY_MOD_SHIFT|DKEY_MOD_CTRL|DKEY_MOD_ALT|DKEY_MOD_KP) // the default modifier mask

// Mouse keys
#define DKEY_MOUSE_KEYS      (0x0800)  // the mouse keys bit

#define DKEY_MOUSE_RIGHT     (1 | DKEY_MOUSE_KEYS)  // the right mouse button
#define DKEY_MOUSE_MIDDLE    (2 | DKEY_MOUSE_KEYS)  // the middle mouse button
#define DKEY_MOUSE_LEFT      (4 | DKEY_MOUSE_KEYS)  // the left mouse button

//
// The DKey class implements methods for storing keyboard keys. 
// 
// @example
// #include <stdio.h>
// #include "ofc/DKey.h"
// 
// int main(int argc, char *argv[])
// {
//   DKey  *key1 = [DKey alloc];
//   DKey  *key2 = [DKey new  ];
//   DText *str;
// 
//   [key1 init :DKEY_ESCAPE];            // Init with the escape key
// 
//                                        // Check for control key
//   printf("Key1 %s a control key.\n", ([key1 isCtrlKey] ? "is" : "is not"));
// 
//   str = [key1 toText];                 // Convert the key to string description
//   printf("Key1 description: %s.\n", [str cstring]);
//   [str free];
// 
// 
//   [key2 set :DKEY_DELETE|DKEY_MOD_CTRL|DKEY_MOD_ALT]; // Set the key to Ctrl-Alt-Del
// 
//   printf("Key2 %s a control key.\n", ([key2 isCtrlKey ] ? "is" : "is not"));
//   printf("Key2 %s a shift key.\n",   ([key2 isShiftKey] ? "is" : "is not"));
//   printf("Key2 %s a alt key.\n",     ([key2 isAltKey  ] ? "is" : "is not"));
// 
//   str = [key2 toText];                 // Convert the key to string description
//   printf("Key2 description:%s.\n", [str cstring]);
//   [str free];
// 
//   [key1 free];                         // Cleanup
//   [key2 free];
//   
//   return 0;
// }
// 

@interface DKey : Object <DTextable,DParsable>
{
@private
   int                       _code;   // the keyboard code
}


//// Constructors

//
// Initialise an empty key
//
// @return the object
//
- (DKey *) init;

//
// Initialise a key
// 
// @param code         the key code
// 
// @return the object
// 
- (DKey *) init :(int) code;


//// Destructor

//
// Free the key
//
// @return the object
//
- free;


//// Member methods

//
// Check if the key is a control key. Note Ctrl-@ till Ctrl-_ are *NOT* 
// control keys.
// 
// @return is it ?
//
- (BOOL) isCtrlKey;

//
// Check if the key is a function key
// 
// @return is it ?
// 
- (BOOL) isFunctionKey;

//
// Check if the key is a keypad key
// 
// @return is it ?
// 
- (BOOL) isKeypadKey;

//
// Check if the key is an alt key
// 
// @return is it ?
// 
- (BOOL) isAltKey;

//
// Check if the key is a shifted key
// 
// @return is it ?
// 
- (BOOL) isShiftKey;

//
// Check if the key is a mouse key
// 
// @return is it ?
// 
- (BOOL) isMouseKey;


//// Main methods

//
// Set the key in the object. (the code is not checked for unknown keys, 
// so 'other' keys can also be stored in the object)
// 
// @param code         the key code 
//
// @return the object
//
- (DKey *) set :(int) code;

//
// Get the key code (incl. modifiers) from the object
//  
// @return the key code
//
- (int) get;

//
// Get the key (without the modifiers)
// 
// @return the key
// 
- (int) key;

//
// Get the modifiers (without the key, no mouse modifier)
// 
// @return the modifiers
// 
- (int) mod;


//// Textable protocol implementation

//
// Convert the key to a text string
//
// @return a (new) text string with the key
//
- (DText *) toText;


//// Parsable protocol implementation

//
// Parse a string to set a key object.
// Example: ctrl-alt-delete, shift-f1, ctrl-I, #
//
// @param cstr     the string to be parsed (moved to first non-parsable char)
// 
// @return the result (0, ERANGE, ENODATA)
//
- (int) fromString :(char **) cstr;

@end
#endif
