#ifndef _DLONG_H
#define _DLONG_H

// This file was automatically generated by objch.

//==============================================================================
//
//            DLong - the long number class in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-02 05:58:14 $ $Revision: 1.11 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DComparable.h"
#include "ofc/DTextable.h"
#include "ofc/DDatable.h"
#include "ofc/DParsable.h"

#include "ofc/DText.h"
#include "ofc/DData.h"

//
// The long class implements a number of methods for storing and manipulating long numbers.
//
// @example
// #include <stdio.h>
// #include "ofc/DLong.h"
// 
// int main(int argc, char *agv[])
// {
//   DLong *l1 = [DLong alloc];
//   DLong *l2 = [DLong new  ];
//   DText *str;
// 
//   [l1 init :-70];                      // Init with a number
// 
//   printf("Long1 has value:%ld.\n", [l1 get]); // Get value from object
// 
//   [l2 set :140];                       // Set with a number
// 
//   str = [l2 toText];
//   printf("Long2 has value:%s as string.\n", [str cstring]); // Convert value to string
//   [str free];
// 
//   if ([l1 compare :l2] == 0)           // Compare longs
//     printf("Long1 and long2 are equal.\n");
//   else if ([l1 compare :l2] < 0)
//     printf("Long1 is smaller than long2.\n");
//   else
//     printf("Long1 is greater than long2.\n");
// 
//                                        // Conversion big- and little-endian
//   printf("Long1 as little-endian (%ld) and as big-endian (%ld).\n", [l1 toLittleEndian], [l2 toBigEndian]);
// 
//   [l1 free];                           // Cleanup
//   [l2 free];
// 
//   return 0;
// }
//

@interface DLong : Object <DComparable,DTextable,DDatable,DParsable>
{
@private
  long        _nr;
}


//// Constructors

//
// Initialises to zero number
//
// @return the object
//
- (DLong *) init;

//
// Initialise to a number
//
// @param number   the number
//
// @return the object
//
- (DLong *) init :(long) number;


//// Member methods

//
// Get the long number
//
// @return the number
//
- (long) get;

//
// Set the long number
//
// @param number  the long
//
// @return the object
//
- (DLong *) set :(long) number;


//// Comparable protocol

//
// Compare two long objects
//
// @param other    the object to compare with
//
// @return the result of the comparison (-1,0,1)
//
- (long) compare :(DLong *) other;


//// Parsable protocol

//
// Parse a string for setting the object
//
// @param cstr     the string to be parsed (moved to first non-parsable char)
//
// @return the result (0, ERANGE, ENODATA)
//
- (int) fromString :(char **) cstr;


//// Textable protocol

//
// Return a text string 
//
// @return a (new) text string with the number
//
- (DText *) toText;


//// Datable protocol

//
// Return a data string
//
// @return a (new) data string
//
- (DData *) toData;


//// Endian methods

//
// Return the BigEndian (network order) order for the object
//
// @return the big endian converted long
//
- (long) toBigEndian;

#ifndef WORDS_BIGENDIAN
#endif
//
// Return the LittleEndian order for the object
//
// @return the little endian converted long
//
- (long) toLittleEndian;

#ifdef WORDS_BIGENDIAN
#endif
@end
#endif
