#ifndef _DPNGIMAGE_H
#define _DPNGIMAGE_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DPNGImage - the PNG image class in the ofc-library
//
//               Copyright (C) 2007  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-20 17:16:29 $ $Revision: 1.8 $
//
//==============================================================================
#ifdef HAVE_DPNGIMAGE
#endif

#include "ofc/config.h"

#include "ofc/DImage.h"
#include "ofc/DDataReadable.h"
#include "ofc/DDataWritable.h"

#if defined(HAVE_LIBPNG) && defined(HAVE_PNG_H)
#define HAVE_DPNGIMAGE 1
#endif

#ifdef HAVE_DPNGIMAGE

//
// The DPNGImage class implements methods for reading and writing PNG images.
// 
// @example
// #include <stdio.h>
// #include "ofc/DPNGImage.h"
// #include "ofc/DFile.h"
// 
// int main(int argc, char *argv[])
// {
//   DPNGImage *image = [DPNGImage new];
//   DFile     *file  = [DFile new];
// 
//   char       name[] = "ofc.png";
// 
//   if ([file open :name :"rb"])
//   {
//     if ([DPNGImage isImage :file])
//     {
//       if ([image open :file])
//       {
//         printf("\"%s\" is a PNG image with width:%d height:%d BytesPerPixel:%d Images:%d\n",
//             name, [image width], [image height], [image bytesPerPixel], [image images]);
//         
//         [image close];
//       }
//       else
//         printf("Could not open image \"%s\".\n", name);
//     }
//     else
//       printf("\"%s\" is not a PNG image.\n", name);
// 
//     [file close];
//   }
//   else
//     printf("Could not find \"%s\":%d\n", name, [file error]);
// 
//   [image free];                        // Cleanup
//   [file  free];
//   
//   return 0;
// }
// 

@interface DPNGImage : Object <DImage>
{
@private
  void                *_png;           // the png pointer
  void                *_info;          // the png info pointer
  unsigned             _height;        // the height of the image
  unsigned             _width;         // the width of the image
  unsigned             _bpp;           // the bytes per pixel
  unsigned             _images;        // the number of images in the image
  int                  _bbc;           // the bits per color
  int                  _colorType;     // the color type
  int                  _interlaceType; // the interlace type
  BOOL                 _reading;       // reading or writing an image ?
  BOOL                 _error;         // is an error active ?
}

#endif
#ifdef HAVE_DPNGIMAGE
//
// Initialise a PNG image object
//
// @return the object
//
- (DPNGImage *) init;


//// Copy related methods

//
// Do a shallow copy of a PNG image (not implemented)
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the image object
// 
// @return the object
// 
- free;


//// Image members

//
// Return the height of the image
// 
// @return the height
// 
- (unsigned) height;

//
// Return the width of the image
// 
// @return the width
// 
- (unsigned) width;

//
// Return the bytes per pixel (3: RGB or 4: RGBA)
// 
// @return the number of bytes
// 
- (unsigned) bytesPerPixel;

//
// Return the number of images in the image (interlacing, always 1)
// 
// @return the images
// 
- (unsigned) images;


//// Image methods

//
// Check if an image is an PNG image
// 
// @param source        the source of the image
// 
// @return is it ?
// 
+ (BOOL) isImage :(id <DDataReadable>) source;

//
// Open the PNG image object for reading
// 
// @param source        the source with the image
//
// @return success
//
- (BOOL) open :(id <DDataReadable>) source;

//
// Open the PNG image object for writing
// 
// @param destination        the file for the image
// @param width              the width of the image
// @param height             the height of the image
// @param bytesPerPixel      the number of bytes per pixel (3 for RGB or 4 for RGBA)
//
// @return succes
//
- (BOOL) open :(id <DDataWritable>) destination :(unsigned) width :(unsigned) height :(unsigned) bytesPerPixel;

//
// Get the next row of image data from the file (format: RGB... or RGBA...)
// 
// @param data          the image data for the next line (size = _width * _bpp)
//
// @return the number of pixels in the row (or 0 for error)
//
- (unsigned) readRow :(unsigned char *) data;

//
// Set the next row of image data in the png file (format: RGB.. or RGBA..)
// 
// @param data          the image data for the next line (size = _width * _bpp)
// @param width         the number of pixels in the row
//
// @return success
//
- (BOOL) writeRow :(unsigned char *) data :(unsigned) width;

//
// Close the image object
// 
// @return success
//
- (BOOL) close;

@end
#endif
#endif
