#ifndef _DPROPERTYTREE_H
#define _DPROPERTYTREE_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DPropertyTree - the property tree class in the ofc-library
//
//               Copyright (C) 2004  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-13 17:14:32 $ $Revision: 1.12 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DText.h"

#include "ofc/DParsable.h"
#include "ofc/DTextable.h"

#include "ofc/DTextReadable.h"
#include "ofc/DTextWritable.h"

//
// The DProperty class implements methods for using properties.
// 

@interface DProperty : Object
{
@private
   DText                    *_name;   // the group or property name
   id <DTextable,DParsable>  _value;  // the reference to the value 
}


//// Constructors

//
// Initialise an empty property
//
// @return the object
//
- (DProperty *) init;

//
// Initialise a property
// 
// @param name         the property name
// @param value        the reference to the value object
// 
// @return the object
// 
- (DProperty *) init :(const char *) name :(id <DTextable,DParsable>) value;

//
// Initialise a group of properties
// 
// @param name         the group name
// 
// @return the object
// 
- (DProperty *) init :(const char *) name;


//// Copy related methods

//
// Do a shallow copy of the property
//
// @return the object
//
- shallowCopy;


//// Destructor

//
// Free the property
//
// @return the object
//
- free;


//// Member methods

//
// Check if the property is a group 
// 
// @return is it ?
//
- (BOOL) isGroup;

//
// Return the name (of the property or group)
// 
// @return the property name (or NULL)
// 
- (const char *) name;

//
// Return the reference to the value object
// 
// @return the reference (or nil)
// 
- (id <DTextable,DParsable>) value;


//// Main methods

//
// Set the property name and reference object for the property
// 
// @param name         the property name
// @param value        the reference object
//
// @return the object
//
- (DProperty *) property :(const char *) name :(id <DTextable,DParsable>) value;

//
// Set the group name for a property group
//  
// @param name      the name of the group
//
// @return the object
//
- (DProperty *) group :(const char *) name;

//
// Compare (case sensitive) the name of the property
//  
// @param name      the name 
//
// @return the result (-1,0,1)
//
- (int) ccompare :(const char *) name;

@end

#include "ofc/DXML.h"

#ifdef HAVE_DXML
#define HAVE_DPROPERTYTREE
#endif
  
#ifdef HAVE_DPROPERTYTREE

//
// The DPropertyTree class implements methods for storing and retrieving
// properties in a xml file. There are methods for adding, removing, importing 
// and exporting properties. 
// 
// @example
// #include <stdio.h>
// #include "ofc/DPropertyTree.h"
// #include "ofc/DFile.h"
// #include "ofc/DText.h"
// #include "ofc/DInt.h"
// #include "ofc/DBool.h"
// 
// int main(int argc, char *argv[])
// {
//   DPropertyTree *tree = [DPropertyTree new];
//   DProperty     *group;
//   DFile         *file = [DFile new];
// 
//   DText         *stringValue = [DText new];  // Property value destinations
//   DInt          *intValue    = [DInt  new];
//   DBool         *boolValue   = [DBool new];
// 
//   char           name[]      = "property.xml";  
//                                              // Build the property tree
//   printf("Building the property tree.\n");
//   group = [tree group :nil :"Example"];      //   First: Add a group to the root
// 
//   [stringValue set :"ExampleString"];        //   Set defaults to the values
//   [intValue    set :10];
//   [boolValue   set :YES];
//                                              //   Second: Add the properties to the group
//   [tree property :group :"string" :stringValue];
//   [tree property :group :"int"    :intValue   ];
//   [tree property :group :"bool"   :boolValue  ];
//                                              // Write the defaults to a property file
//   printf("Saving property tree to \"%s\".\n", name);
//   if ([file open :name :"w"])
//   {
//     if (![tree write :file :name])
//       printf("Could not write \"%s\".\n", name);
// 
//     [file close];
//   }
//   else
//     printf("Could \"%s\" not open for writing:%d\n", name, [file error]);
// 
//                                        // Test: reset the properties
//   printf("Reset the properties.\n");
//   [stringValue set :""];
//   [intValue    set :0 ];
//   [boolValue   set :NO];
//                                        // Read the property file
//   printf("Read the properties back from \"%s\".\n", name);
//   if ([file open :name :"r"])
//   {
//     if (![tree read :file :name])
//       printf("Could not read \"%s\".\n", name);
// 
//     [file close];
//   }
//   else 
//     printf("Could \"%s\" not open for reading:%d\n", name, [file error]);
// 
//   printf("Property \"string\":%s\n", [stringValue cstring]);
//   printf("Property \"int\"   :%d\n", [intValue get]);
//   printf("Property \"bool\"  :%d\n", [boolValue get]);
// 
//   [tree free];                         // Cleanup
//   [file free];
// 
//   return 0;
// }
// 

@interface DPropertyTree : Object <DXMLHandler>
{
@private
  DTree                    *_properties; // the property tree
  DTreeIterator            *_iter;       // the property tree iterator
  DProperty                *_root;       // the root of the properties
  DXMLReader               *_reader;     // the xml reader
}

#endif
#ifdef HAVE_DPROPERTYTREE

//// Constructors

//
// Initialise an empty property tree
//
// @return the object
//
- (DPropertyTree *) init;

//
// Initialise a named property tree
//
// @param name      the name of the property tree
// 
// @return the object
//
- (DPropertyTree *) init :(const char *) name;


//// Copy related methods

//
// Do a shallow copy of the property tree (not implemented)
//
// @return the object
//
- shallowCopy;


//// Destructor

//
// Free the property tree
//
// @return the object
//
- free;


//// Member methods

//
// Get the name of the tree
// 
// @return the name of the tree
// 
- (const char *) name;

//
// Set the name of the tree
// 
// @param name      the name of the tree
// 
// @return the object
// 
- (DPropertyTree *) name :(const char *) name;


//// Main methods

//
// Add a group to the property tree
//
// @param parent   the parent (group) property 
// @param name     the group name
//
// @return the (new) group property (or nil)
//
- (DProperty *) group :(DProperty *) parent :(const char *) name;

//
// Add a property to the tree of properties
// 
// @param parent   the parent (group) property (or nil for root)
// @param name     the property name
// @param value    the reference to the value object
// 
// @return the (new) property (or nil if parent not found)
// 
- (DProperty *) property :(DProperty *)              parent 
                         :(const char *)             name
                         :(id <DTextable,DParsable>) value;

//
// Import the properties
// 
// @param source   the properties source
// @param name     the name of the source
// 
// @return success
// 
- (BOOL) read :(id <DTextReadable>) source :(const char *) name;

//
// Export the properties
//
// @param destination the destination for the property tree 
// @param name        the name of the destination
// 
// @return success
// 
- (BOOL) write :(id <DTextWritable>) destination :(const char *) name;

//
// Remove a property from the property tree
// 
// @param property   the property to be removed (and FREED)
// 
// @return success (property is only freed if succesfull)
// 
- (BOOL) remove :(DProperty *) property;

//
// Remove a property from the property tree
// 
// @param parent   the parent (group) property
// @param name     the name of the property
// 
// @return success (property is freed if succesfull)
// 
- (BOOL) remove :(DProperty *) parent :(const char *) name;


//// Property methods (private methods)

//
// Add an property/group to the tree of properties
// 
// @param parent    the parent property
// @param property  the property to be added
// 
// @return is the parent property found ?
// 
- (BOOL) _addProperty :(DProperty *) parent :(DProperty *) property;

//
// Find a property
// 
// @param parent   the group of the property
// @param name     the name of the property
// 
// @return the property (or nil if not found)
// 
- (DProperty *) _findProperty :(DProperty *) parent :(const char *) name;

//
// Remove a property
// 
// @param property  the property to be removed
// 
// @return is the property removed ?
// 
- (BOOL) _removeProperty :(DProperty *) property;


//// DXMLHandler protocol implementation (private methods)

//
// Process the start of a document
// 
// @param version     the version number
// @param encoding    the encoding
// @param standalone  is the xml document standalone ? (-1 = missing)
// 
// @return success
// 
- (BOOL) startDocument :(const DXMLChar *) version :(const DXMLChar *) encoding :(int) standalone;

//
// Process the end of a document
// 
// @return success
// 
- (BOOL) endDocument;

//
// Process the start of an element
// 
// @param name     the name of the element
// 
// @return success
// 
- (BOOL) startElement :(const DXMLChar *) name;

//
// Process an attribute for an element (after a startElement)
// 
// @param attribute  the attribute
// @param value      the value for the attribute
// 
// @return success
// 
- (BOOL) attribute :(const DXMLChar *) attribute :(const DXMLChar *) value;

//
// Process the end of an element
// 
// @return success
//
- (BOOL) endElement;

//
// Process a string of characters
// 
// @param text     the text of the characters
// 
// @return success
// 
- (BOOL) characters :(const DXMLChar *) text;

//
// Process a comment
// 
// @param text     the text of the comment
// 
// @return success
// 
- (BOOL) comment :(const DXMLChar *) text;

//
// Process a processing instruction
// 
// @param target   the target
// @param value    the value
// 
// @return success
// 
- (BOOL) processingInstruction :(const DXMLChar *) target :(const DXMLChar *) value;

//
// Process the start of a CDATA section
// 
// @return success
// 
- (BOOL) startCDATA;

//
// Process the end of a CDATA section
// 
// @return success
// 
- (BOOL) endCDATA;

//
// Start of a namespace declaration
// 
// @param prefix   the prefix
// @param uri      the uri
// 
// @return success
// 
- (BOOL) startNamespace :(const DXMLChar *) prefix :(const DXMLChar *) uri;

//
// End of a namespace declaration
// 
// @return success
// 
- (BOOL) endNamespace;

//
// Process unparsed text in the xml document (e.g. the DTD)
// 
// @param text     the unparsed text
// 
// @return success
// 
- (BOOL) unparsed :(const DXMLChar *) text;

// 
// An error is found during the parsing of the file
// 
// @param number       the error number
// @param name         the name of the config file
// @param lineNumber   the line number in the config file
// @param columnNumber the column number in the config file
// 
// @return success
// 
- (void) error :(int) number :(const char *) name :(int) lineNumber :(int) columnNumber;

@end
#endif
#endif
