#ifndef _DSIMPLEFONT_H
#define _DSIMPLEFONT_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DSimpleFont - a simple, bitmap font in the ofc-library
//
//               Copyright (C) 2007  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-26 05:50:30 $ $Revision: 1.5 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DFont.h"


//
// The DSimpleFont class implements a simple 8x8 bitmap font. It is used as a start
// font. It only implements the characters space till ~, all others are translated
// to a '?'.
// 
// @example
// #include <stdio.h>
// #include "ofc/DSimpleFont.h"
// 
// int main(int argc, char *argv[])
// {
//   DSimpleFont *font = [DSimpleFont new];
//   DFT_Glyph    glyph;
// 
//   unsigned width,height;
//                                        // Print font info
//   printf("Family name:%s\n",    [font familyName]);
//   printf("Style name:%s\n",     [font styleName]);
//   printf("Glyphs in font:%d\n", [font glyphsInFont]);
//   
//   [font stringSize :"hello" :&width :&height]; // Get the string dimensions
//   printf("String size of \"hello\":%u - %u\n", width, height);
// 
//   if ([font glyph :'a' :&glyph])       // Render the font to a glyph
//   {
//     unsigned xe = glyph.bitmapWidth -1;
//     unsigned ye = glyph.bitmapHeight - 1;
//     unsigned x,y;
//     
//     unsigned char *bitmap = glyph.bitmap;
//    
//     printf("Glyph bitmap:\n");         // Print the bitmap of the glyph
//     for (y = 0; y <= ye; y++)
//     {
//       unsigned char  mask = 0x80;
//       unsigned char *line = bitmap;
//       
//       for (x = 0; x <= xe; x++)
//       {
//         printf("%c", ((*line & mask) ? '1' : '0'));
// 
//         mask >>= 1;
//         if (mask == 0)
//         {
//           mask = 0x80;
//           line++;
//         }
//       }
//       bitmap += glyph.bitmapPitch;
//       printf("\n");
//     }
//   }
//   else
//     printf("The glyph 'a' is not present in the font.\n");
// 
//   [font free];
// 
//   return 0;
// }
// 

@interface DSimpleFont : Object <DFont>
{
}

//
// Initialise a simple bitmap font
//
// @return the object
//
- (DSimpleFont *) init;


//// Copy related methods

//
// Do a shallow copy of a simple bitmap font
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the font
// 
// @return the object
// 
- free;


//// Font members

//
// Get the family name of the font
//
// @return the family name (or NULL)
//
- (const char *) familyName;

//
// Get the style name of the font
//
// @return the style name (or NULL)
//
- (const char *) styleName;

//
// Get the number of characters in font
//
// @return the number of characters in the font (or -1)
//
- (int) glyphsInFont;


//// Font methods

//
// Set the font size based on the width and height of one character
// 
// @param width         the requested width
// @param height        the requested height
//
// @return success
//
- (BOOL) size :(unsigned) width :(unsigned) height;

//
// Set the font size based on a string and requested a size
// 
// @param cstr          the string
// @param width         the requested width
// @param height        the requested height
//
// @return success
//
- (BOOL) size :(const char *) cstr :(unsigned) width :(unsigned) height;

//
// Get the size of a string for the current font size
// 
// @param cstr          the string
// @param width         the width of the string
// @param height        the height of the string
//
// @return success
//
- (BOOL) stringSize :(const char *) cstr :(unsigned *) width :(unsigned *) height;

//
// Get the size of a character for the current font and size
// 
// @param ch            the character
// @param width         the width of the string
// @param height        the height of the string
//
// @return success
//
- (BOOL) charSize :(char) ch :(unsigned *) width :(unsigned *) height;

//
// Get the glyph info for blitting a character for the current font and size
// 
// @param ch            the character
// @param glyph         the character bitmap and blit info
//
// @return success
//
- (BOOL) glyph :(char) ch :(DFT_Glyph *) glyph;

@end
#endif
