#ifndef _DSIZEABLE_H
#define _DSIZEABLE_H

//==============================================================================
//
//    DSizeable - the protocol for sizeable classes in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-07-19 12:28:59 $ $Revision: 1.3 $
//
//==============================================================================

// The DSizeable protocol specifies methods for insuring the size of the data
// in an object and the amount of increase if the size is extended. See 
// @class(DText) and @class(DData).

@protocol DSizeable

//
// Insure the size for the data in the object
//
// @param size     the requested size for the data
//
// @return the object
//

- size :(unsigned long) size;

//
// Set the amount of size increase if the size must be extended
//
// @param extra    the amount (def. 1)
//
// @return the object
//

- extra :(unsigned) extra;

@end

#endif

