#ifndef _DSOCKETADDRESS_H
#define _DSOCKETADDRESS_H

// This file was automatically generated by objch.

//==============================================================================
//
//     DSocketAddress - the socket address wrapper classes in the ofc-library
//
//               Copyright (C) 2004  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-11 16:59:46 $ $Revision: 1.20 $
//==============================================================================
#ifndef WIN32
#endif
#ifdef WIN32
#endif

#include "ofc/config.h"

#include "ofc/DText.h"

#ifdef WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/un.h>
#endif


#define DSA_AF_INET    (AF_INET)         // Inet socket family
#ifndef WIN32
#ifdef  AF_UNIX
#define DSA_AF_UNIX    (AF_UNIX)         // Unix socket family
#endif
#endif
#ifdef  AF_INET6
#define DSA_AF_INET6   (AF_INET6)        // IPv6 socket family
#endif

//
// The DSocketAddressable protocol specifies the methods for a socket address
// 

@protocol DSocketAddressable

//
// Copy the object
// 
// @return the object
// 

- copy;

//
// Free the address
//
// @return the object
//

- free;

//
// Return the last error
// 
// @return the last error
// 

- (int) error;
  
//
// Return the family of the socket address
// 
// @return the family
// 
- (int) family;

//
// Return the socket address
// 
// @return the socket address
// 
- (void *) sockaddr;

//
// Return the size of the socket address
// 
// @return the size
// 
- (int) size;

//
// Return the port (network format)
// 
// @return the port
// 
- (int) port;

//
// Return the host name
// 
// @return a (new) text object with the host name
// 
- (DText *) host;

//
// Set the socket address with an external socket address
// 
// @param addr      the socket address
// @param size      the size of the socket address
// 
// @return success
// 

- (BOOL) sockaddr :(void *) addr :(int) size;

//
// Close the socket address
// 
// @return the object
// 

- close;

@end

  
//
// The DInetSocketAddress class implements a number of methods for using 
// inet socket addresses.
// 
// @example
// #include <stdio.h>
// #include "ofc/DSocketAddress.h"
// 
// int main(int argc, char *argv[])
// {
//   DInetSocketAddress *addr1 = [DInetSocketAddress new];
//   unsigned char       b1,b2,b3,b4;
// 
//   [addr1 host :"localhost" :27];       // Set the host of the inet address
// 
//   printf("Family of address1: %d\n", [addr1 family]); // Print the family of the address
//   printf("Port of address1: %d\n",   [addr1 port  ]); // Print the port of the address
//   
//   [addr1 get :&b1 :&b2 :&b3 :&b4];
//   printf("Inet of address1: %d.%d.%d.%d \n", b1, b2, b3, b4); // Print the inet of the address
// 
//   [addr1 free];
// 
//   return 0;
// }
// 

@interface DInetSocketAddress : Object <DSocketAddressable>
{
@private
  struct sockaddr     _address;    // the socket address
  struct sockaddr_in *_inet;       // the cast to inet socket address
  int                 _error;      // the last error (for host method)
}


//// Constructors

//
// Initialise the socket address
//
// @return the object
//
- (DInetSocketAddress *) init;

//
// Initialise the socket address with an inet address
// 
// @param address   the inet address
// @param port      the port
//
// @return the object
//
- (DInetSocketAddress *) init :(long) address :(int) port;

//
// Initialise the socket address with an inet address (b1.b2.b3.b4)
// 
// @param b1        the first  byte of the inet address
// @param b2        the second byte of the inet address
// @param b3        the third  byte of the inet address
// @param b4        the fourth byte of the inet address
// @param port      the port
//
// @return the object
//
- (DInetSocketAddress *) init :(unsigned char) b1 :(unsigned char) b2 :(unsigned char) b3 :(unsigned char) b4 :(int) port;


//// Copy related methods

//
// Copy the object (for protocol)
// 
// @return the object
// 
- copy;

//
// Do a shallow copy of the object
// 
// @return the object
// 
- shallowCopy;


//// Deconstructor

//
// Free the socket object
//
// @return the object
//
- free;


//// Member services

//
// Return the last error
// 
// @return the last error
// 
- (int) error;

//
// Return the family of the socket address
// 
// @return the family
// 
- (int) family;

//
// Return the socket address
// 
// @return the socket address
// 
- (void *) sockaddr;

//
// Return the size of the socket address
// 
// @return the size
// 
- (int) size;

//
// Return the port (network format)
// 
// @return the port
// 
- (int) port;

//
// Set the port 
// 
// @param port     the port
// 
// @return the object
// 
- (DInetSocketAddress *) port :(int) port;

//
// Return the host name 
// 
// @return a (new) text object with the host name (or nil for not found)
// 
- (DText *) host;

#if defined(WIN32) || defined(__FreeBSD__)
#endif

//// Socket address setters

//
// Set the socket address with an inet address
//
// @param address   the inet address
// @param address   the port of the address
//
// @return the object
//
- (DInetSocketAddress *) set :(unsigned long) address :(int) port;

//
// Set the socket address with an inet address
// 
// @param b1        the first  byte of the inet address
// @param b2        the second byte of the inet address
// @param b3        the third  byte of the inet address
// @param b4        the fourth byte of the inet address
// @param port      the port of the address
//
// @return the object
//
- (DInetSocketAddress *) set :(unsigned char) b1 :(unsigned char) b2 :(unsigned char) b3 :(unsigned char) b4 :(int) port;

//
// Get the inet address from the socket address 
// 
// @param b1        the first  byte of the inet address
// @param b2        the second byte of the inet address
// @param b3        the third  byte of the inet address
// @param b4        the fourth byte of the inet address
//
// @return the port (host format)
//
- (int) get :(unsigned char *) b1 :(unsigned char *) b2 :(unsigned char *) b3 :(unsigned char *) b4;

//
// Set the socket address with an inet host
// 
// @param host      the host name for the inet address
// @param port      the port for the address
// 
// @return the object
// 
- (BOOL) host :(const char *) name :(int) port;

//
// Set the socket address with an external socket address
// 
// @param addr      the socket address
// @param size      the size of the socket address
// 
// @return success
// 
- (BOOL) sockaddr :(void *) addr :(int) size;

//
// Set the loopback address for an inet socket address
// 
// @param port      the port
// 
// @return the object
// 
- (DInetSocketAddress *) loopback :(int) port;

//
// Set the any address for the inet socket address
// 
// @param port      the port
// 
// @return the object
// 
- (DInetSocketAddress *) any :(int) port;

//
// Set the broadcast address for an inet socket address
// 
// @param port      the port
// 
// @return the object
// 
- (DInetSocketAddress *) broadcast :(int) port;

//
// Close the socket address
// 
// @return the object
// 
- (DInetSocketAddress *) close;

@end

#ifdef DSA_AF_UNIX
//
// The DUnixSocketAddress class implements a number of methods for using 
// unix (file) socket addresses.
// 
// @example 
// #include <stdio.h>
// #include "ofc/DSocketAddress.h"
// 
// int main(int argc, char *argv[])
// {
// #ifdef DSA_AF_UNIX
//   DUnixSocketAddress *addr2 = [DUnixSocketAddress alloc]; 
// 
//   [addr2 init :"udp1"];                // Init with a filename
// 
//   printf("Family of address2: %d\n", [addr2 family]); // Print the family of the unix address
// 
//   [addr2 free];
// #else
//   printf("UnixSocketAddress not present.\n");
// #endif
// 
//   
//   return 0;
// }
// 

@interface DUnixSocketAddress : Object <DSocketAddressable>
{
@private
  struct sockaddr     _address;    // the address
  struct sockaddr_un *_unix;       // the unix cast to address
  int                 _size;       // the size of sockaddr
  int                 _error;      // always 0
}

#endif
#ifdef DSA_AF_UNIX

//// Constructors

//
// Initialise the socket address
//
// @return the object
//
- (DUnixSocketAddress *) init;

//
// Initialise the socket address with a filename
//
// @param filename  the name of the file
//
// @return the object
//
- (DUnixSocketAddress *) init :(const char *) filename;


//// Copy related methods

//
// Copy the object (for protocol)
// 
// @return the object
// 
- copy;

//
// Do a shallow copyo of the object
// 
// @return the object
// 
- shallowCopy;


//// Deconstructor

//
// Free the address object
//
// @return the object
//
- free;


//// Member services

//
// Return the last error
// 
// @return the last error
// 
- (int) error;

//
// Return the family of the socket address
// 
// @return the family
// 
- (int) family;

//
// Return the socket address
// 
// @return the socket address
// 
- (void *) sockaddr;

//
// Return the size of the socket address
// 
// @return the size
// 
- (int) size;

//
// Return the port
// 
// @return the port
// 
- (int) port;

//
// Return the filename
// 
// @return a (new) DText object with the filename (or nil for not found)
// 
- (DText *) host;

//
// Close the socket address
// 
// @return the object
// 
- (DUnixSocketAddress *) close;


//// Socket address setters

//
// Set the filename for an unix socket
// 
// @param filename  the name of the file 
// @param port      the port for the socket
// 
// @return success
// 
- (BOOL) filename :(const char *) name;

//
// Set the socket address with an external socket address
// 
// @param addr      the socket address
// @param size      the size of the socket address
// 
// @return success
// 
- (BOOL) sockaddr :(void *) addr :(int) size;

@end
#endif

#ifdef DSA_AF_INET6

//
// The DInet6SocketAddress class implements a number of methods for using 
// IPv6 socket addresses.
// 

@interface DInet6SocketAddress : Object <DSocketAddressable>
{
@private
  struct sockaddr_in6  _inet6;      // the inet6 socket address
  int                  _error;      // the last error (for host method)
}
#endif
#ifdef DSA_AF_INET6

//// Constructors

//
// Initialise the socket address
//
// @return the object
//
- (DInet6SocketAddress *) init;

//
// Initialise the socket address with an inet6 address
// 
// @param a1        the first  32 bit of the inet address
// @param a1        the second 32 bit of the inet address
// @param a1        the third  32 bit of the inet address
// @param a1        the fourth 32 bit of the inet address
// @param port      the port
// @param flowInfo  the flow info
// @param scopeId   the scope id
//
// @return the object
//
- (DInet6SocketAddress *) init :(unsigned long) a1 :(unsigned long) a2 :(unsigned long) a3 :(unsigned long) a4 :(int) port :(unsigned long) flowInfo :(unsigned long) scopeId;

//
// Initialise the socket address with an inet6 address
// 
// @param address   the 16 bytes of the inet address
// @param port      the port
// @param flowInfo  the flow info
// @param scopeId   the scope id
//
// @return the object
//
- (DInet6SocketAddress *) init :(unsigned char[16]) address :(int) port :(unsigned long) flowInfo :(unsigned long) scopeId;


//// Copy constructor

//
// Copy the object (for protocol)
// 
// @return the object
// 
- copy;


//// Deconstructor

//
// Free the socket object
//
// @return the object
//
- free;


//// Member services

//
// Return the last error
// 
// @return the last error
// 
- (int) error;

//
// Return the family of the socket address
// 
// @return the family
// 
- (int) family;

//
// Return the socket address
// 
// @return the socket address
// 
- (void *) sockaddr;

//
// Return the size of the socket address
// 
// @return the size
// 
- (int) size;

//
// Return the port (network format)
// 
// @return the port
// 
- (int) port;

//
// Set the port 
// 
// @param port     the port
// 
// @return the object
// 
- (DInet6SocketAddress *) port :(int) port;

//
// Return the host name 
// 
// @return a (new) text object with the host name (or nil for not found)
// 
- (DText *) host;

#if defined(WIN32) || defined(__FreeBSD__)  
#endif

//// Socket address setters

//
// Set the socket address with an inet6 address
//
// @param a1        the first  32 bit of the inet address
// @param a2        the second 32 bit of the inet address
// @param a3        the third  32 bit of the inet address
// @param a4        the fourth 32 bit of the inet address
// @param port      the port
// @param flowInfo  the flow info
// @param scopeId   the scope id
//
// @return the object
//
- (DInet6SocketAddress *) set :(unsigned long) a1 :(unsigned long) a2 :(unsigned long) a3 :(unsigned long) a4 :(int) port :(unsigned long) flowInfo :(unsigned long) scopeId;

//
// Get the socket address
//
// @param a1        the first  32 bit of the inet address
// @param a2        the second 32 bit of the inet address
// @param a3        the third  32 bit of the inet address
// @param a4        the fourth 32 bit of the inet address
// @param port      the port
// @param flowInfo  the flow info
// @param scopeId   the scope id
//
// @return the object
//
- (DInet6SocketAddress *) get :(unsigned long *) a1 :(unsigned long *) a2 :(unsigned long *) a3 :(unsigned long *) a4 :(int *) port :(unsigned long *) flowInfo :(unsigned long *) scopeId;

//
// Set the socket address with an inet6 address
// 
// @param address   the 16 bytes of the inet6 address
// @param port      the port of the address
// @param flowInfo  the flow info
// @param scopeId   the scope id
//
// @return the object
//
- (DInet6SocketAddress *) set :(unsigned char[16]) address :(int) port :(unsigned long) flowInfo :(unsigned long) scopeId;

//
// Get the socket address
// 
// @param address   the 16 bytes of the inet6 address
// @param port      the port of the address
// @param flowInfo  the flow info
// @param scopeId   the scope id
//
// @return the object
//
- (DInet6SocketAddress *) get :(unsigned char[16]) address :(int *) port :(unsigned long *) flowInfo :(unsigned long *) scopeId;

//
// Get the socket address (16 bit)
// 
// @param address   the 8 16-bit values of the inet6 address
// @param port      the port of the address
// @param flowInfo  the flow info
// @param scopeId   the scode id
// 
// @return the object
// 
- (DInet6SocketAddress *) get16 :(unsigned[8]) address :(int *) port :(unsigned long *) flowInfo :(unsigned long *) scopeId;

//
// Set the socket address with an inet6 host
// 
// @param host      the host name for the inet address
// @param port      the port for the address
// @param flowInfo  the flow info
// @param scopeId   the scope id
// 
// @return the object
// 
- (BOOL) host :(const char *) name :(int) port :(unsigned long) flowInfo :(unsigned long) scopeId;

//
// Set the socket address with an external socket address
// 
// @param addr      the socket address
// @param size      the size of the socket address
// 
// @return success
// 
- (BOOL) sockaddr :(void *) addr :(int) size;

//
// Set the loopback address for an inet6 socket address
// 
// @param port      the port
// @param flowInfo  the flow info
// @param scopeId   the scope id
// 
// @return the object
// 
- (DInet6SocketAddress *) loopback :(int) port :(unsigned long) flowInfo :(unsigned long) scopeId;

#ifdef WIN32
#endif
//
// Set the any address for the inet6 socket address
// 
// @param port      the port
// @param flowInfo  the flow info
// @param scopeId   the scope id
// 
// @return the object
// 
- (DInet6SocketAddress *) any :(int) port  :(unsigned long) flowInfo :(unsigned long) scopeId;

#ifdef WIN32
#endif
//
// Close the socket address
// 
// @return the object
// 
- (DInet6SocketAddress *) close;

@end
#endif
#endif
