#ifndef _DSTACKABLE_H
#define _DSTACKABLE_H

//==============================================================================
//
//       DStackable - the protocol for a collection that acts as a stack
//
//               Copyright (C) 2004  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-07-19 12:28:59 $ $Revision: 1.3 $
//
//==============================================================================

// The DStackable protocol specifies methods for classes that implements
// a stack.


@protocol DStackable

//
// Return the number of stored objects
// 
// @return the length
// 
- (long) length;

//
// Check if the buffer is empty
// 
// @return is it ?
// 
- (BOOL) isEmpty;

//
// Push an object on the stack (LIFO)
//
// @param obj      the object to be pushed
//
// @return success (or buffer full)
//
- (BOOL) push :(id) obj;

//
// Pop an object from the stack (LIFO)
//
// @return the object (or nil if empty)
//
- (id) pop;

//
// Return the object on top of the stack (LIFO)
// (without changing the top of the stack)
//
// @return the object (or nil if empty)
//
- (id) tos;

@end

#endif

