#ifndef _DSYSTEMLOGGER_H
#define _DSYSTEMLOGGER_H

// This file was automatically generated by objch.

//==============================================================================
//
//          DSystemLogger - the system logger in the ofc-library
//
//               Copyright (C) 2004  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-07 17:38:49 $ $Revision: 1.5 $
//
//==============================================================================
#ifdef HAVE_DSYSTEMLOGGER
#endif

#include "ofc/config.h"

#include "ofc/DLogger.h"

#ifdef HAVE_SYSLOG_H
#define HAVE_DSYSTEMLOGGER
#endif

#ifdef HAVE_DSYSTEMLOGGER
//
// The DSystemLogger class implements methods for continuous logging
// to the system logger. On Unix this will be the syslog deamon. On
// Win32 there is not an usable logger present. So this class will only
// be present on Unix.
// 
// @example
// #include <stdio.h>
// #include "ofc/DSystemLogger.h"
// #include "ofc/DLog.h"
// 
// int main(int argc, char *argv[])
// {
// #ifdef HAVE_DSYSTEMLOGGER
//   DSystemLogger *logger = [DSystemLogger alloc];
//   int            i;
// 
//   [logger init :"example" :NO];        // Init the logger with application "example" and logging to stderr
// 
//   dlogger(logger);                     // Give tot the base logger the system logger as destination
// 
//   dlog(DLOG_ALERT, "Alert!!");         // Log an alert
// 
//   for (i = 0; i < 10; i++)             // Add a repeated error message
//   {
//     dlog(DLOG_ERROR, "Errors");
//   }
// 
//   dlogmask(DLOG_NOTICE|DLOG_DEBUG);    // Mask some log levels
//     
//   dlog(DLOG_WARNING, "Warning..Masked"); // Masked warning
//   dlog(DLOG_DEBUG,   "Debug:%d",7);    // Debug message
//   dlog(DLOG_NOTICE,  "Notice");        // Notice message
// 
//   dlogger(nil);                        // Stop logging via the system logger
// 
//   [logger free];                       // Cleanup
// 
//   printf("Events succesfully sent to the system logger.\n");
// #else
//   printf("SystemLogger not present in the library.\n");
// #endif
// 
//   return 0;
// }
// 

@interface DSystemLogger : Object <DLogger>
{
  int _mask;        // the (copy of) the mask
}
#endif
#ifdef HAVE_DSYSTEMLOGGER

//// Constructors

//
// Initialise a default system logger
//
// @return the object
//
- (DSystemLogger *) init;

//
// Initialise a system logger
// 
// @param appName   the application name (or NULL)
// @param toStdErr  also logging to stderr ?
//
// @return the object
//
- (DSystemLogger *) init :(const char *) appName :(BOOL) toStdErr;


//// Copy constructor

//
// Do a shallow copy of the object (not implemented)
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the object
//
// @return the object
//
- free;


//// Implementation of interface DLogger

//
// Log an event by the logger
// 
// @param level      the level of the event
// @param message    the message to be logged
// @param parameters the optional parameters for the message
// 
// @return success
// 
- (BOOL) doLog :(int) level :(const char *) message;

//
// Set the active levels
// 
// @param level      the bitwise or'ed active levels 
// 
// @return the previous mask
// 
- (int) mask :(int) levels;

//
// Set the active level range
// 
// @param high        the highest level to mask
// @param low         the lowest level to mask
// 
// @return the previous mask
//
- (int) mask :(int) high :(int) low;

@end
#endif
#endif
