#ifndef _DTELNETCLIENT_H
#define _DTELNETCLIENT_H

// This file was automatically generated by objch.

//==============================================================================
//
//          DTelNetClient - the telnet client class in the ofc-library
//
//               Copyright (C) 2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-24 05:37:06 $ $Revision: 1.10 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DURL.h"
#include "ofc/DSocket.h"


#define DTNC_PORT     (23)

// Telnet commands
#define DTNC_IAC              (255)  // Interpret as command
#define DTNC_DONT             (254)  // Do not use this option (sender does not want receiver to use the option)
#define DTNC_DO               (253)  // Do use this option (sender wants receiver to use the option)
#define DTNC_WONT             (252)  // Will not use this option (sender will not use the option)
#define DTNC_WILL             (251)  // Will use this option (sender will use the option)
#define DTNC_SB               (250)  // Start subnegotiation
#define DTNC_GA               (249)  // Go ahead
#define DTNC_EL               (248)  // Erase line
#define DTNC_EC               (247)  // Erase character
#define DTNC_AYT              (246)  // Are you there
#define DTNC_AO               (245)  // Abort output
#define DTNC_IP               (244)  // Interrupt process
#define DTNC_BRK              (243)  // Break
#define DTNC_DM               (242)  // Data mark
#define DTNC_NOP              (241)  // No operation
#define DTNC_SE               (240)  // End subnegotiation

// Telnet options
#define DTNC_BINARY           (0)    // 8-bit data path
#define DTNC_ECHO             (1)    // Echo
#define	DTNC_RCP              (2)    // Prepare to reconnect
#define	DTNC_SGA              (3)    // Suppress go ahead
#define	DTNC_NAMS             (4)    // Approximate message size
#define	DTNC_STATUS           (5)    // Give status
#define	DTNC_TM               (6)    // Timing mark
#define	DTNC_RCTE             (7)    // Remote controlled transmission and echo
#define DTNC_NAOL             (8)    // Negotiate about output line width
#define DTNC_NAOP             (9)    // Negotiate about output page size
#define DTNC_NAOCRD           (10)   // Negotiate about CR disposition
#define DTNC_NAOHTS           (11)   // Negotiate about horizontal tabstops
#define DTNC_NAOHTD           (12)   // Negotiate about horizontal tab disposition
#define DTNC_NAOFFD           (13)   // Negotiate about formfeed disposition
#define DTNC_NAOVTS           (14)   // Negotiate about vertical tab stops
#define DTNC_NAOVTD           (15)   // Negotiate about vertical tab disposition
#define DTNC_NAOLFD           (16)   // Negotiate about output LF disposition
#define DTNC_XASCII           (17)   // Extended ascii character set
#define	DTNC_LOGOUT           (18)   // Force logout
#define	DTNC_BM               (19)   // Byte macro
#define	DTNC_DET              (20)   // Data entry terminal
#define	DTNC_SUPDUP           (21)   // Supdup protocol
#define	DTNC_SUPDUPOUTPUT     (22)   // Supdup output
#define	DTNC_SNDLOC           (23)   // Send location
#define	DTNC_TTYPE            (24)   // Terminal type
#define	DTNC_EOR              (25)   // End or record
#define	DTNC_TUID             (26)   // TACACS user identification
#define	DTNC_OUTMRK           (27)   // Output marking
#define	DTNC_TTYLOC           (28)   // Terminal location number
#define	DTNC_3270REGIME       (29)   // 3270 regime
#define	DTNC_X3PAD            (30)   // X.3 PAD
#define	DTNC_NAWS             (31)   // Window size
#define	DTNC_TSPEED           (32)   // Terminal speed
#define	DTNC_LFLOW            (33)   // Remote flow control
#define DTNC_LINEMODE         (34)   // Linemode option
#define DTNC_XDISPLOC         (35)   // X Display Location
#define DTNC_OLD_ENVIRON      (36)   // Old - Environment variables
#define	DTNC_AUTHENTICATION   (37)   // Authenticate
#define	DTNC_ENCRYPT          (38)   // Encryption option
#define DTNC_NEW_ENVIRON      (39)   // New - Environment variables
#define	DTNC_EXOPL            (255)  // Extended-options-list

#define DTNC_NEGOTIATE        (1)    // Option is under negotiation (initiated by the client)
#define DTNC_SERVER           (2)    // Option state for the server
#define DTNC_CLIENT           (4)    // Option state for the client

#define DTNC_OPTIONS          (256)  // the number of options

//
// The DTelNetClient class implements a telnet client. The class supports
// negotiation of options. Also subnegotiation is supported. There is not 
// yet support for interrupt sending.
//
// @example
// #include <stdio.h>
// #include "ofc/DTelNetClient.h"
// 
// @interface MyClient : DTelNetClient    // MyClient processes the special server responses 
// {
// }
// 
// - (BOOL) processOpenNegotiation :(int) who :(int) option :(int) state;
// - (BOOL) processResponseNegotiation :(int) who :(int) option :(BOOL) accepted;
// - (BOOL) processRequestSubNegotiation :(int) option;
// - (BOOL) processSpecialCommand :(unsigned char) command;
// @end
// 
// @implementation MyClient
// 
// - (BOOL) processOpenNegotiation :(int) who :(int) option :(int) state;
// {
//   printf("Opening negotiation option %s for %s state %d\n", 
//       [DTelNetClient optionToString :option], (who == DTNC_SERVER ? "Server" : "Client"), state);
//   
//   return (option == 24 ? YES : NO);    // Accept option 24, refuse others
// }
// 
// - (BOOL) processResponseNegotiation :(int) who :(int) option :(BOOL) accepted;
// {
//   printf("Server accepted negotiation option %s for %s state %d\n", 
//       [DTelNetClient optionToString :option], (who == DTNC_SERVER ? "Server" : "Client"), accepted);
//   
//   return YES;
// }
// 
// - (BOOL) processRequestSubNegotiation :(int) option
// {
//   printf("Server request the value for the sub negotiation of option %s\n", 
//       [DTelNetClient optionToString :option]);
//   
//   if (option == 24) 
//     [self respondSubNegotiation :option :"dec-vt100" :9]; // Send value subnegotiation
//   
//   return YES;
// }
// 
// - (BOOL) processSpecialCommand :(unsigned char) command;
// {
//   printf("Special command: %s received\n", [DTelNetClient commandToString :command]);
//   
//   return YES;
// }
// @end
// 
// int main(int argc, char *argv[])
// {
//   MyClient           *client  = [MyClient new];
//   DInetSocketAddress *address = [DInetSocketAddress new];
//   DURL          *url          = [DURL new];
//   DData         *response     = nil;
// 
//   [address host :"localhost" :DTNC_PORT];
//   
//   [client open :address];              // Open connection to the server
//   
//   if ([client isConnected])
//   {
//     char buffer[256];
//     
//     buffer[0] = EOS;
//                                        // Open the negotiations
//     [client requestOpenNegotiation :DTNC_SERVER :DTNC_TM   :YES];
//     
//     do
//     {
//       BOOL input = NO;
//       BOOL empty = NO;
//       
//       do
//       {                                // Send user input to server
//         [client sendText :buffer];
// 
//         if (!input)                    // Receive responses
//           response = [client receive];
//         
//         buffer[0] = EOS;
//         
//         if (response != nil)
//         {
//           long i;
//           long length = [response length];
//       
//           printf("Response:");         // Print response
//           for (i = 0; i < length; i++)
//             printf("%c", [response get :i]);
//           
//           if (length > 0)
//             input = ([response get :(length-1)] != '\n');
//           
//           empty = (length == 0);
//           
//           [response free]; response = nil;
//         }
//       }
//       while (([client pendingRequests]) || (empty) || (!input));
//       
//       fflush(stdout);
//                                        // Ask input from user
//       fgets(buffer, sizeof(buffer), stdin);
//     }                                  // Keep asking till 'exit'
//     while (strncmp(buffer, "exit", 4) != 0);
//       
//     [client close];
//   }
//   else
//     printf("Could not connect to telnet server.\n");
//   
//   [client  free];                      // Cleanup
//   [url     free];
//   [address free];
// }
// 

@interface DTelNetClient : Object
{
@private
  DSocket            *_client;                 // the client socket
  BOOL                _connected;              // is the client connected ?
  int                 _options[DTNC_OPTIONS];  // the negotiated state op the options
  DData              *_request;                // the request to be sent to the server
}


//// Constructors

//
// Initialise the telnet client
//
// @return the object
//
- (DTelNetClient *) init;


//// Deconstructor

//
// Free the telnet client object
//
// @return the object
//
- free;


//// Copy related methods

//
// Do a shallow copy of the object (not implemented)
//
//
- shallowCopy;


//// Class methods

//
// Return a string describing a command
// 
// @param command   the command
// 
// @return the describing string (or NULL)
// 
+ (const char *) commandToString :(unsigned char) command;

//
// Return a string describing an option
// 
// @param option    the option
// 
// @return the describing string (or NULL)
// 
+ (const char *) optionToString :(int) option;


//// Member methods

//
// Return the local client socket object
// 
// @return a reference to the client socket object
//
- (DSocket *) socket;

//
// Check if there is a connection
// 
// @return is there a connection with the server ?
// 
- (BOOL) isConnected;

//
// Check the state of an option (for the server or client)
// 
// @param who      the side (DTNC_SERVER or DTNC_CLIENT)
// @param option   the option to be checked
// 
// @return the state 
// 
- (BOOL) option :(int) who :(int) option;


//// Connection methods

//
// Open the connection to the telnet server
// 
// @param server       the telnet server
// 
// @return success
// 
- (BOOL) open :(id <DSocketAddressable>) server;

//
// Close the connection to the telnet server
// 
// @return success
// 
- (BOOL) close;


//// Sending methods

//
// Put the open negotiation for an option message request in the send buffer.
// 
// @param who      the side the option has to implement (DTNC_SERVER, DTNC_CLIENT)
// @param option   the option to be negotiated
// @param state    the state for the option
// 
// @return success
// 
- (BOOL) requestOpenNegotiation :(int) who :(int) option :(BOOL) state;

//
// Put the request for Sub Negotiation message in the send buffer
// 
// @param option   the option to be requested
// 
// @return success
// 
- (BOOL) requestSubNegotiation :(int) option;

//
// Put the resonse Sub Negotiation message in the send buffer
// 
// @param option   the option to be responded
// @param data     the data for the sub negotiation
// @param length   the length of the data
// 
// @return success
// 
- (BOOL) respondSubNegotiation :(int) option :(unsigned char *) data :(int) length;

//
// Put the negotiation response message in the request buffer.
//
// @param who      the side that implements the option
// @param option   the option 
// @param accepted is the option accepted ?
// 
// @return success
// 
- (BOOL) respondNegotiation :(int) who :(int) option :(BOOL) accepted;

//
// Put Are You There in the send buffer
// 
// @return success
// 
- (BOOL) AYT;

//
// Put (normal) text in the send buffer and send the buffer to the server
// 
// @param text     the text for the server (or NULL)
// 
// @return success
// 
- (BOOL) sendText :(const unsigned char *) text;

//
// Check if the send buffer has pending messages (requests or responses)
// 
// @return are there ?
// 
- (BOOL) pendingRequests;

//
// Check if there are pending negotiations with the server
// 
// @return are there ?
// 
- (BOOL) pendingNegotiations;


//// Receiving methods

//
// Wait (blocking) for a response of the server. Depending on the contents of
// the response, the methods processOpenNegotiation, processSubNegotiation 
// and/or processSpecialCommand will be called.
//
// @return a (new) DData object with the received data (without the (sub) 
// negotiations, nil for error) 
// 
- (DData *) receive ;

//
// Wait (blocking) for a response of the server. Depending on the contents of
// the response, the methods processOpenNegotiation, processSubNegotiation 
// and/or processSpecialCommand will be called. The remaining data will be 
// placed in the response object.
//
// @param  response  the DData object to be filled with the received data
// 
// @return success
// 
- (BOOL) receive :(DData *) response;

// 
// Process the receiving of a special command (to be overridden)
//
// @param command  the received (telnet) command
// 
// @return success
// 
- (BOOL) processSpecialCommand :(unsigned char) command;

//
// Process the receiving of the open negotiation request from the server (to be overridden, default: not supported)
//
// @param who       the side that implements the option (DTNC_SERVER, DTNC_CLIENT)
// @param option    the option to be negotiated
// @param state     the suggested state for the option
// 
// @return is the option supported ?
// 
- (BOOL) processOpenNegotiation :(int) who :(int) option :(int) state;

//
// Process the receiving of an accepted negotiation response from the server (to be overridden)
// 
// @param who       the side that implements the option (DTNC_SERVER, DTNC_CLIENT)
// @param option    the option that is negotiated with the server
// @param accepted  is the option accepted ?
// 
// @return success
// 
- (BOOL) processResponseNegotiation :(int) who :(int) option :(BOOL) accepted;

//
// Process the request for a sub negotiation from the server. Use 
// sendResponseSubNegotiation to reply to this request (to be overridden)
// 
// @param option   the option in the request for sub negotiation
//
// @return success
// 
- (BOOL) processRequestSubNegotiation :(int) option;

//
// Process the response for a sub negotiation from the server (to be overridden)
// 
// @param option   the option that was sub negotiated
// @param data     the data of the sub negotiation
// @param length   the length of the data
// 
// @return success
// 
- (BOOL) processResponseSubNegotiation :(int) option :(const unsigned char *) data :(int) length;


//// Message processing methods (private methods)

//
// Scan the received message
// 
// @param data     the received data
// @param length   the length of the data
// 
// @return a (new) DData object with the received data (or nil for no data)
//
- (BOOL) _scanMessage :(DData *) response :(const unsigned char *) data :(int) length;

//
// Scan a negotiation message
// 
// @param data     the received data
// @param length   the length of the data
// 
// @return success
//
- (BOOL) _scanNegotiation :(const unsigned char **) data :(int *) length;

//
// Scan a sub negotiation message
// 
// @param data     the received data
// @param length   the length of the data
// 
// @return success
//
- (BOOL) _scanSubNegotiation :(const unsigned char **) data :(int *) length;

@end
#endif
