#ifndef _DTEXTREADABLE_H
#define _DTEXTREADABLE_H

//==============================================================================
//
//         DTextReadable - the text readable protocol in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-07-19 12:28:59 $ $Revision: 1.5 $
//
//==============================================================================

#include "ofc/config.h"

@class DText;

// The DTextReadable protocol specifies the interface for reading of text from
// a class implementing this interface.

@protocol DTextReadable

//
// Read text string (all available text)
//
// @return a (new) text string
//

- (DText *) readText;

//
// Read text string 
//
// @param length   the length of the text to be read
//
// @return a (new) text string
//

- (DText *) readText :(long) length;

//
// Read a line of text (till Eof or \n)
//
// @return a (new) text string
//

- (DText *) readLine;

//
// Read a character
//
// @return the character read
//

- (char) readChar;

//
// Tell the current position in the readable
//
// @return the current position
//

- (unsigned long) tell;

//
// Move the current position in the readable
//
// @param offset   the offset from the origin (in bytes)
// @param origin   the origin for the offset (0=begin, 1=current, 2=end)
//
// @return success
//

- (BOOL) seek :(unsigned long) offset :(int) origin;

//
// Skip a number of positions
//
// @param offset   the number of bytes to skip
//
// @return success
//

- (BOOL) skip :(unsigned long) offset;

//
// Is the end of the readable reached ?
//
// @return is it?
//

- (BOOL) isEof;

//
// Return the last error
//
// @return the last error (or 0)
// 

- (int) error;

@end

#endif

/*============================================================================*/

