#ifndef _DTEXTSCREEN_H
#define _DTEXTSCREEN_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DTextScreen - the text screen wrapper class in the ofc-library
//
//               Copyright (C) 2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-09-06 19:24:32 $ $Revision: 1.18 $
//
//==============================================================================
#ifdef WIN32
#endif

#include "ofc/config.h"

#include "ofc/DColor.h"

#include "ofc/DScreenHandler.h"
#include "ofc/DDrawable.h"

#ifdef WIN32
# define HAVE_DTEXTSCREEN (1)
#else
# if defined(HAVE_NCURSES_H) && defined(HAVE_LIBNCURSES)
#  define HAVE_DTEXTSCREEN (1)
# endif
#endif



#ifdef HAVE_DTEXTSCREEN

//
// The DTextDrawable class implements a class with the shared methods between
// the text screen class and the text surface class. It contains the methods
// for drawing, clipping, color.. The x and y positions start from zero, so the
// screen size is 0..maxX and 0..maxY (inclusive).
// ToDo: crossings between drawn lines, font class, drawing boxes, circles..
// 

@interface DTextDrawable : Object <DDrawable>
{
@protected
  unsigned             _maxX;        // the right x-position of the drawable
  unsigned             _maxY;        // the bottom y-position of the drawable
  unsigned             _attribute;   // the current color attributes

@private
  unsigned             _clipMinX;    // the left x-position for the clip area
  unsigned             _clipMaxX;    // the right x-position for the clip area
  unsigned             _clipMinY;    // the top y-position for the clip area
  unsigned             _clipMaxY;    // the bottom y-position for the clip area
  unsigned             _cursorX;     // the x-position of the cursor
  unsigned             _cursorY;     // the y-position of the cursor
  BOOL                 _drawing;     // is drawing active ?
  DColor              *_fgc;         // the foreground color
  DColor              *_bgc;         // the background color
  
  int                  _lineType;    // the last line type
}

#endif
#ifdef HAVE_DTEXTSCREEN
//
// Initialise to an text drawable object
//
// @return the object
//
- (DTextDrawable *) init;


//// Copy related methods

//
// Do a shallow copy of the text drawable object
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the text screen
// 
// @return the object
// 
- free;


//// Cursor methods

//
// Return the x position of the cursor
// 
// @return the x position
// 
- (unsigned) cursorX;

//
// Return the y position of the cursor
// 
// @return the y position
//
- (unsigned) cursorY;

//
// Set the cursor to a position
// 
// @param x     the x-position
// @param y     the y-position
// 
// @return success
// 
- (BOOL) cursor :(unsigned) x :(unsigned) y;

//
// Set the visibility of the cursor
// 
// @param state   the cursor state (DDW_CURSOR..)
// 
// @return the previous state of the cursor
// 
- (int) cursor :(int) state;

//
// Set the visibility of the mouse pointer
// 
// @param state   the pointer state (DDW_POINTER..)
// 
// @return the previous state of the pointer
// 
- (int) pointer :(int) state;

//
// Return the last error
// 
// @return the last error (0)
// 
- (int) error;

//
// Tell the current position in the writeable
//
// @return the current position
//
- (unsigned long) tell;

//
// Move the current position to a location
//
// @param offset   the offset from the origin (in bytes)
// @param origin   the origin for the offset (0=begin, 1=current, 2=end)
//
// @return success
//
- (BOOL) seek :(unsigned long) offset :(int) origin;

//
// Skip a number of positions
//
// @param offset   the number of bytes to skip
//
// @return success
//
- (BOOL) skip :(unsigned long) offset;


//// Surface methods

//
// Return the maximum x-position
// 
// @return the maximum x-position
//
- (unsigned) maxX;

//
// Return the number of lines
// 
// @return the number of lines
// 
- (unsigned) lines;

//
// Return the maximum y-position
// 
// @return the maximum y-position
// 
- (unsigned) maxY;

//
// Return the number of columns
// 
// @return the number of columns
// 
- (unsigned) columns;

//
// Set the dimensions of the drawable
// 
// @param columns the number of columns (> 0)
// @param lines   the number of lines (> 0)
// 
// @return the object
// 
- (DTextDrawable *) drawable :(unsigned) columns :(unsigned) lines;

//
// Check if a cursor position is valid (within drawable)
// 
// @param x     the x-position
// @param y     the y-position
// 
// @return is it ?
// 
- (BOOL) isValid :(unsigned) x :(unsigned) y;


//// Clipping methods

//
// Set a clipping area
// 
// @param minX    the left x-position of the clip area
// @param maxX    the right x-position of the clip area
// @param minY    the top y-position of the clip area
// @param maxY    the bottom y-position of the clip area
// 
// @return success
// 
- (BOOL) clip :(unsigned) minX :(unsigned) minY :(unsigned) maxX :(unsigned) maxY;

//
// Reset the clipping area to the full surface
// 
// @return success
// 
- (BOOL) clip;

//
// Return the top position of the clip area
// 
// @return the top y-position
// 
- (unsigned) clipMinY;

//
// Return the bottom position of the clip area
// 
// @return the bottom y-position
// 
- (unsigned) clipMaxY;

//
// Return the left position of the clip area
// 
// @return the left x-position
// 
- (unsigned) clipMinX;

//
// Return the right position of the clip area
// 
// @return the right x-position
// 
- (unsigned) clipMaxX;


//// Color methods

//
// Set the fore- and background color
// 
// @param fgc     the foreground color
// @param bgc     the background color
// 
// @return success
// 
- (BOOL) color :(DColor *) fgc :(DColor *) bgc;

//
// Set the foreground color
// 
// @param fgc     the foreground color
// 
// @return success
// 
- (BOOL) foregroundColor :(DColor *) fgc;

//
// Get the foreground color
// 
// @return a reference to the foreground color
// 
- (DColor *) foregroundColor;

//
// Set the background color
// 
// @param bgc     the background color
// 
// @return success
// 
- (BOOL) backgroundColor :(DColor *) bgc;

//
// Get the background color
// 
// @return a reference to the background color
// 
- (DColor *) backgroundColor;


//// Drawing control methods

//
// Start drawing on the screen
// 
// @return success
// 
- (BOOL) startDrawing;

//
// Start drawing in a clipped area on the screen
// 
// @param minX      the left x-position of the clipping area
// @param maxX      the right x-position of the clipping area
// @param minY      the top y-position of the clipping area
// @param maxY      the bottom y-position of the clipping area
// 
// @return success
//
- (BOOL) startDrawing :(unsigned) minX :(unsigned) minY :(unsigned) maxX :(unsigned) maxY;

//
// Stop drawing on the screen
// 
// @return the screen object
// 
- (DTextDrawable *) stopDrawing;

//
// Check if the screen is in drawing mode
// 
// @return is it ?
// 
- (BOOL) isDrawing;


//// Drawing methods

//
// Clear the drawable
// 
// @return success
// 
- (BOOL) clear;

//
// Write a text string on the screen on the current cursor position
// 
// @param text    the text string
// 
// @return success
// 
- (BOOL) writeText :(const char *) text;

//
// Write a character on the screen on the current cursor position
// 
// @param ch      the character
// 
// @return success
// 
- (BOOL) writeChar :(char) ch;

//
// Write a text line on the screen on the current cursor position
// 
// @param text    the text string
// 
// @return success
// 
- (BOOL) writeLine :(const char *) text;

//
// Write a text string on the screen
// 
// @param startX  the start x position
// @param startY  the start y position
// @param text    the text string
// 
// @return success
// 
- (BOOL) writeText :(unsigned) startX :(unsigned) startY :(const char *) text;

//
// Write a character on the screen
// 
// @param startX  the start x position
// @param startY  the start y position
//
// @param ch      the character
// 
// @return success
// 
- (BOOL) writeChar :(unsigned) startX :(unsigned) startY :(char) ch;

//
// Read a character from the drawable
// 
// @param startX  the start x position
// @param startY  the start y position
// 
// @return the character (or -1)
// 
- (int) readChar :(unsigned) startX :(unsigned) startY;

//
// Draw a (horizontal) line on the drawable
//
// @param startX    the x position for the start of the line
// @param endX      the x position for the end of the line
// @param startY    the y position for the line
// @param lineType  the type of the line DDW_LINE_...
//
// @return success
//
- (BOOL) drawHLine :(unsigned) startX :(unsigned) startY :(unsigned) endX :(int) lineType;

//
// Draw a horizontal line with the previous line width and type settings
//
// @param startX    the x position for the start of the line
// @param endX      the x position for the end of the line
// @param startY    the y position for the line
//
// @return success
//
- (BOOL) drawHLine :(unsigned) startX :(unsigned) startY :(unsigned) endX ;

//
// Draw a (horizontal) line on the drawable starting from the current position and
// with the previous line width and type settings
//
// @param endX      the x position for the end of the line
//
// @return success
//
- (BOOL) drawHLine : (unsigned) endX ;

//
// Draw a vertical line on the screen
// 
// @param startX      the x position for the line
// @param startY      the y position for the line
// @param endY        the y position for the end of the line
// @param lineType    the type of the line (DDW_LINE_...)
// 
// @return success
// 
- (BOOL) drawVLine :(unsigned) startX :(unsigned) startY :(unsigned) endY :(int) lineType;

//
// Draw a vertical line with the current line type and width settings
// 
// @param startX      the x position for the line
// @param startY      the y position for the line
// @param endY        the y position for the end of the line
// 
// @return success
// 
- (BOOL) drawVLine :(unsigned) startX :(unsigned) startY :(unsigned) endY;

//
// Draw a (vertical) line on the drawable starting from the current position and
// with the current line width and type settings
//
// @param endY      the y position for the end of the line
//
// @return success
//
- (BOOL) drawVLine :(unsigned) endY;

//
// Draw a line on the screen
//
// @param startX      the x position for the start of the line 
// @param endX        the x position for the end of the line
// @param startY      the y position for the start of the line
// @param endY        the y position for the end of the line
// @param lineType    the type of the line (DDW_LINE_...)
// 
// @return success
// 
- (BOOL) drawLine :(unsigned) startX :(unsigned) startY :(unsigned) endX :(unsigned) endY :(int) lineType;

//
// Draw a line with the current line type and width settings
//
// @param startX  the x position for the start of the line 
// @param endX    the x position for the end of the line
// @param startY  the y position for the start of the line
// @param endY    the y position for the end of the line
// 
// @return success
// 
- (BOOL) drawLine :(unsigned) startX :(unsigned) startY :(unsigned) endX :(unsigned) endY;

//
// Draw a line on the drawable starting from the current position and
// with the current line width and type settings
//
// @param endX      the x-position for the end of the line
// @param endY      the y-position for the end of the line
//
// @return success
//
- (BOOL) drawLine :(unsigned) endX :(unsigned) endY;

//
// Draw a point on the drawable
// 
// @param startX   the x-position
// @param startY   the y-position
// 
// @return success
// 
- (BOOL) drawPoint :(unsigned) startX :(unsigned) startY;

//
// Draw a point on the current position
// 
// @return success
// 
- (BOOL) drawPoint;

//
// Blit another text drawable on this text drawable
// 
// @param startX    the start x-position where the info is placed on this drawable
// @param startY    the start y-position where the info is placed on this drawable
// @param other     the other drawable
// @param oStartX   the start x-position on the other drawable
// @param oEndX     the end x-position on the other drawable
// @param oStartY   the start y-position on the other drawable
// @param oEndY     the end y-position on the other drawable
// 
// @return success
// 
- (BOOL) blit :(unsigned) startX :(unsigned) startY 
              :(DTextDrawable *) other :(unsigned) oStartX :(unsigned) oStartY :(unsigned) oEndX :(unsigned) oEndY;


//// Private methods

//
// Put a character on the screen (internal use, to be overridden)
//
// @param x       the x-position
// @param y       the y-position 
// @param ch      the character
// @param attr    the attributes
// 
// @return success
// 
- (BOOL) _putChar :(unsigned) x :(unsigned) y :(int) ch :(unsigned) attr;

//
// Read a character from the screen (internal use, to be overridden)
// 
// @param x       the x-position
// @param y       the y-position
// @param ch      the read character
// @param attr    the read attributes
// 
// @return success
// 
- (BOOL) _getChar :(unsigned) x :(unsigned) y :(int *) ch :(unsigned *) attr;

@end
#endif

#ifdef HAVE_DTEXTSCREEN

//
// The DTextScreen class implements a class to use the text screen. On unix it
// is a wrapper for the ncurses library. For obvious reasons there can be only
// one text screen object in any application.
// 
// @example
// #include <stdio.h>
// #include "ofc/DTextScreen.h"
// 
// #ifdef HAVE_DTEXTSCREEN 
// @interface MyScreen : DTextScreen <DScreenHandler>  // Class for the screen and its events
// {
//   DColor *fgc;
//   DColor *bgc;
// }
// - (MyScreen *) init;
// - (BOOL) open;
// - (void) draw :(const char *) key;
// - (BOOL) keyPress :(DKey *) key :(int) state;      // Callbacks
// - (BOOL) mouseMove :(unsigned) x :(unsigned) y;
// - (BOOL) mouseButtonPress :(DKey *) button :(int) state;
// - (BOOL) screenResize :(unsigned) maxX :(unsigned) maxY;
// - (BOOL) screenFocus :(BOOL) focus;
// - (BOOL) screenRedraw :(unsigned) minX :(unsigned) maxX :(unsigned) minY :(unsigned) maxY;
// @end
// 
// @implementation MyScreen
// 
// - (MyScreen *) init
// {
//   [super init];
//   
//   fgc = [DColor new];
//   bgc = [DColor new];
// 
//   return self;
// }
// 
// - (BOOL) open                          // Open the screen with ..
// {
//   BOOL ok = NO;
// 
//   [fgc textColor :DCLR_YELLOW];        // .. foreground and background color
//   [bgc textColor :DCLR_BLUE];
//   
//   ok = [self open :fgc :bgc];
// 
//   if (ok)
//   {
//     [self screenHandler :self];        // Object is handling its events itself
//   }
// 
//   return ok;
// }
// 
// - (void) draw :(const char *) key      // Screen drawing
// {
//   int maxX = [self maxX];              // Dimensions of screen
//   int maxY = [self maxY];
// 
//   char hello[] = "Hello World";
//   char quit[]  = "Press q to quit";
// 
//   [self startDrawing];                 // Start drawing
//   [self clear];
//   [self drawHLine :1 :1 :maxX-1];      // Draw a some lines
//   [self drawHLine :maxX-1 :maxY-1 :1];
//                                        // Write some text
//   [self cursor :((maxX - strlen(hello))/2) :((maxY-2)/2)];
//   [self writeText :hello];
// 
//   [self cursor :((maxX - strlen(quit))/2) :((maxY+2)/2)];
//   [self writeText :quit];
// 
//   if (key != NULL)
//   {
//     [self cursor :3 :maxY-3];
//     [self writeText :"Key pressed:"];
//     [self writeText :key];
//   }
//   [self stopDrawing];
// }
//                                        // Callback for pressed key
// - (BOOL) keyPress :(DKey *) key :(int) state
// {
//   if ([key key] != 'q')
//   {
//     DText *info = [key toText];
//     
//     [self draw :[info cstring]];
// 
//     [info free];
//   }
//   
//   return ([key key] != 'q');
// }
//                                        // Callback for moved mouse
// - (BOOL) mouseMove :(unsigned) x :(unsigned) y
// {
//   return YES;
// }
//                                        // Callback for pressed mouse button
// - (BOOL) mouseButtonPress :(DKey *) button :(int) state
// {
//   return YES;
// }
//                                        // Callback for screen resize
// - (BOOL) screenResize :(unsigned) maxX :(unsigned) maxY
// {
//   [self draw :NULL];
// 
//   return YES;
// }
//                                        // Callback for lost and gained screen focus
// - (BOOL) screenFocus :(BOOL) focus
// {
//   [self draw :NULL];
// 
//   return YES;
// }
//                                        // Callback for screen redraw
// - (BOOL) screenRedraw :(unsigned) minX :(unsigned) maxX :(unsigned) minY :(unsigned) maxY
// {
//   [self draw :NULL];
// 
//   return YES;
// }
// 
// @end
// #endif
// 
// int main(int argc, char *argv[])
// {
// #ifdef HAVE_DTEXTSCREEN 
//   MyScreen *screen = [MyScreen new];
// 
//   if ([screen open])                   // Open the screen
//   {
//     [screen draw :NULL];               // Draw the screen
// 
//     [screen waitEvents];               // Wait for and process events
//   }
//   else
//     printf("Could not open the screen..\n");
// 
//   [screen free];                       // Cleanup
// #else
//   printf("DTextScreen is not present in the library.\n");
// #endif
//   return 0;
// }
// 

@interface DTextScreen : DTextDrawable
{
@private
  BOOL                 _isOpen;      // is the text screen open ?
  id <DScreenHandler>  _handler;     // the screen event handler
  DKey                *_key;         // the key
  unsigned long        _mouseMask;   // the previous mouse mask
}

#endif
#ifdef HAVE_DTEXTSCREEN

//// Constructors

//
// Initialise to an unopened text screen object
//
// @return the object
//
- (DTextScreen *) init;

//
// Initialise an opened text screen object with a fore- and background color
// 
// @param fgc     the foreground color
// @param bgc     the background color
//
// @return the object
//
- (DTextScreen *) init :(DColor *) fgc :(DColor *) bgc;


//// Copy related methods

//
// Do a shallow copy of the text screen object (not implemented)
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the text screen
// 
// @return the object
// 
- free;


//// Screen status methods

//
// Check if the text screen object is open
// 
// @return is it ?
//
- (BOOL) isOpen;

//
// Check if the terminal support colors
// 
// @return support the terminal colors ?
// 
- (BOOL) hasColors;


//// Event handler methods

//
// Set the screen event handler
// 
// @param handler   the (reference to the) screen handler
// 
// @return the previous screen handler
// 
- (id <DScreenHandler>) screenHandler :(id <DScreenHandler>) handler;

//
// Return the current screen event handler
// 
// @return the (reference to the) screen handler
// 
- (id <DScreenHandler>) screenHandler;


//// Screen control methods

//
// Open the text screen (only one instance is opened)
//
// @param fgc     the foreground color
// @param bgc     the background color
// 
// @return success
// 
- (BOOL) open :(DColor *) fgc :(DColor *) bgc;

#ifdef WIN32
#endif
// 
// Close the text screen
// 
// @return the object
// 
- (DTextScreen *) close;

#ifdef WIN32
#endif

//// Screen output methods

//
// Beep the terminal
//
// @return the object
//
- (DTextScreen *) beep ;

#ifdef WIN32
#endif
//
// Set the visibility of the cursor
// 
// @param state   the cursor state (DDW_CURSOR..)
// 
// @return the previous state of the cursor
// 
- (int) cursor :(int) state;

#ifdef WIN32
#endif
//
// Put a character on the screen (internal use)
//
// @param x       the x-position
// @param y       the y-position 
// @param ch      the character
// @param attr    the attributes
// 
// @return success
// 
- (BOOL) _putChar :(unsigned) x :(unsigned) y :(int) ch :(unsigned) attr;

#ifdef WIN32
#endif
//
// Read a character from the screen (internal use)
// 
// @param x       the x-position
// @param y       the y-position
// @param ch      the read character
// @param attr    the read attributes
// 
// @return success
// 
- (BOOL) _getChar :(unsigned) x :(unsigned) y :(int *) ch :(unsigned *) attr;

#ifdef WIN32
#endif

//// Drawing control methods

//
// Start drawing on the screen
// 
// @return success
// 
- (BOOL) startDrawing;

//
// Stop drawing on the screen
// 
// @return the screen object
// 
- (DTextScreen *) stopDrawing;

#ifndef WIN32
#endif

//// Event processing

//
// Wait (and process) events
// 
// @return the number of processed events
// 
- (unsigned) waitEvents;

#ifndef WIN32
#endif
#ifdef WIN32
#ifdef KEY_RESIZE
#endif
#endif
//
// Process any pending events
//
// @return the number of processed events
// 
- (unsigned) processEvents;

#ifndef WIN32
#endif
#ifdef WIN32
#ifdef KEY_RESIZE
#endif
#endif
//
// Clear any pending events
//
// @return the object
// 
- (DTextScreen *) clearEvents;

#ifdef WIN32
#endif
@end
#endif

#ifdef HAVE_DTEXTSCREEN


#include "ofc/DIntArray.h"


//
// The DTextSurface class implements a class for off-screen drawing to a text
// surface. This surface can then be blit to the text screen.
// 

@interface DTextSurface : DTextDrawable
{
@private
  DIntArray           *_characters;     // the characters in the surface
  DIntArray           *_attributes;     // the attributes in the surface
}

#endif
#ifdef HAVE_DTEXTSCREEN
//
// Initialise to an text surface object
//
// @return the object
//
- (DTextSurface *) init;


//// Copy related methods

//
// Do a shallow copy of the text surface object
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the text surface
// 
// @return the object
// 
- free;


//// Surface control methods

//
// Check if the surface is open
// 
// @return is it ?
// 
- (BOOL) isOpen;

//
// Open a text screen surface 
//
// @param columns   the number of columns 
// @param lines     the number of lines
// @param fgc       the initial foreground color
// @param bgc       the initial backgrounc color
// 
// @return success
// 
- (BOOL) open :(unsigned) columns :(unsigned) lines :(DColor *) fgc :(DColor *) bgc;

#ifdef WIN32
#endif
//
// Close the surface
// 
// @return the object
// 
- (DTextSurface *) close;


//// Private surface output methods

//
// Write a character on the screen (unclipped)
//
// @param x       the x-position
// @param y       the y-position 
// @param ch      the character
// @param attr    the attributes
// 
// @return success
// 
- (BOOL) _putChar :(unsigned) x :(unsigned) y :(int) ch :(unsigned) attr;

//
// Read a character from the screen (internal use)
// 
// @param x       the x-position
// @param y       the y-position
// @param ch      the read character
// @param attr    the read attributes
// 
// @return success
// 
- (BOOL) _getChar :(unsigned) x :(unsigned) y :(int *) ch :(unsigned *) attr;

@end
#endif
#endif
