#ifndef _DTRUETYPEFONT_H
#define _DTRUETYPEFONT_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DTrueTypeFont - the true type font class in the ofc-library
//
//               Copyright (C) 2007  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2008-08-26 05:50:30 $ $Revision: 1.7 $
//
//==============================================================================
#ifdef HAVE_DTRUETYPEFONT
#endif

#include "ofc/config.h"

#include "ofc/DFont.h"

#if defined(HAVE_FREETYPE)
#define HAVE_DTRUETYPEFONT 1
#endif


#ifdef HAVE_DTRUETYPEFONT

//
// The DTrueTypeFont class implements methods for using a TrueType font. 
// Actually every font that can be loaded with the FreeType library, can 
// be used by this class.
// 
// @example
// #include <stdio.h>
// #include "ofc/DTrueTypeFont.h"
// 
// int main(int argc, char *argv[])
// {
//   DTrueTypeFont *font = [DTrueTypeFont new];
//   DFT_Glyph    glyph;
// 
//   unsigned     width,height;
//   char         name[] = "../test/arial.ttf";
// 
//   if ([font open :name])               // Open font file
//   {                                    // Print font info
//     printf("Family name:%s\n",    [font familyName]);
//     printf("Style name:%s\n",     [font styleName]);
//     printf("Glyphs in font:%d\n", [font glyphsInFont]);
// 
//     [font size :"a" :20 :15];          // Give the glyphs a size
// 
//     [font stringSize :"hello" :&width :&height]; // Get string dimensions
//     printf("String size of \"hello\":%u - %u\n", width, height);
// 
//     if ([font glyph :'a' :&glyph])     // Render a character to a glyph
//     {
//       unsigned xe = glyph.bitmapWidth -1;
//       unsigned ye = glyph.bitmapHeight - 1;
//       unsigned x,y;
//       
//       unsigned char *bitmap = glyph.bitmap;
//      
//       printf("Glyph bitmap:\n");       // Print the bitmap of the glyph
//       for (y = 0; y <= ye; y++)
//       {
//         unsigned char  mask = 0x80;
//         unsigned char *line = bitmap;
//         
//         for (x = 0; x <= xe; x++)
//         {
//           printf("%c", ((*line & mask) ? '1' : '0'));
// 
//           mask >>= 1;
//           if (mask == 0)
//           {
//             mask = 0x80;
//             line++;
//           }
//         }
//         bitmap += glyph.bitmapPitch;
//         printf("\n");
//       }
//     }
//     else
//       printf("The glyph 'a' is not present in the font.\n");
// 
//     [font close];
//   }
//   else
//     printf("Could not open font file \"%s\".\n", name);
// 
//   [font free];
// 
//   return 0;
// }
// 

@interface DTrueTypeFont : Object <DFont>
{
@private
  void     *_font;    // the loaded font
}

#endif
#ifdef HAVE_DTRUETYPEFONT

//// Constructors

//
// Initialise an graphic drawable object
//
// @return the object
//
- (DTrueTypeFont *) init;


//// Copy related methods

//
// Do a shallow copy of the graphic drawable object
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the truetype font
// 
// @return the object
// 
- free;


//// Font members

//
// Get the family name of the font
//
// @return the family name (or NULL)
//
- (const char *) familyName;

//
// Get the style name of the font
//
// @return the style name (or NULL)
//
- (const char *) styleName;

//
// Get the number of characters in font
//
// @return the number of characters in the font (or -1)
//
- (int) glyphsInFont;

#if 0
#endif

//// Font methods

//
// Open a (true type) font file
//
// @param filename      the name of the font file
// 
// @return success
//
- (BOOL) open :(const char *) filename;

//
// Check if the font is open
//
// @return is it ?
//
- (BOOL) isOpen;

//
// Close the font (file)
//
// @return success
//
- (BOOL) close;

//
// Set the font size based on the width and height of one character
// 
// @param width         the requested width
// @param height        the requested height
//
// @return success
//
- (BOOL) size :(unsigned) width :(unsigned) height;

//
// Set the font size based on a string and requested a size
// 
// @param cstr          the string
// @param width         the requested width
// @param height        the requested height
//
// @return success
//
- (BOOL) size :(const char *) cstr :(unsigned) width :(unsigned) height;

//
// Get the size of a string for the current font size
// 
// @param cstr          the string
// @param width         the width of the string
// @param height        the height of the string
//
// @return success
//
- (BOOL) stringSize :(const char *) cstr :(unsigned *) width :(unsigned *) height;

//
// Get the size of a character for the current font and size
// 
// @param ch            the character
// @param width         the width of the string
// @param height        the height of the string
//
// @return success
//
- (BOOL) charSize :(char) ch :(unsigned *) width :(unsigned *) height;

//
// Get the glyph info for blitting a character for the current font and size
// 
// @param ch            the character
// @param glyph         the character bitmap and blit info
//
// @return success
//
- (BOOL) glyph :(char) ch :(DFT_Glyph *) glyph;

@end
#endif
#endif
